import 'dart:async';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:http/http.dart';
import 'package:progress_dialog/progress_dialog.dart';
import 'package:rapidfire/Config.dart';
import 'package:rapidfire/Db.dart';
import 'package:rapidfire/GlobalRF.dart';
import 'package:rapidfire/SystemRF.dart';
import 'package:rapidfire/uiux/DialogRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';

import 'dart:convert';
import 'BGHelperDB.dart';
import 'BGSyncSMain.dart';
import 'Dynamic.dart';
import 'GlobalScopeRF.dart';

class Api {
  var db = Db();
  ProgressDialog pdlg;

  var syncRequest = {
    "UserId": 0,
    "TableName": "",
    "Command": "add",
    "TotalRecord": 0,
    "PageNo": 0,
    "PageSize": 0,
    "ApiPacket": {
      "Packet": null,
      "PacketList": null,
    }
  };

  syncModels(String jobName, BuildContext context, Function dispatcher, [List<String> tables]) {
    if (dispatcher != null) {
      _syncWithService(jobName, context, dispatcher, null); // pass table
    } else {
      _syncAll(); // pass table
    }
  }

  _syncWithService(
      String jobName, BuildContext context, Function dispatcher, Map<String, dynamic> inputData) async {
    pdlg = ProgressDialog(
      context,
      type: ProgressDialogType.Normal,
      isDismissible: true,
    );
    resetService(jobName);
    await pdlg.show();
    startService(jobName, dispatcher, inputData);
    onSyncComplete(jobName);
  }

  runSyncService(Function reflectionAgent, IConfigRF models) async {
    reflectionAgent.call();
    Configuration _configuration = new Configuration();
    models.configure(_configuration);
    Models modelList = new Models();
    _configuration.model.appModels.register(modelList);

    for (var entry in modelList.get().entries) {
      var modelRF = entry.value as ModelRF;
      var _syncStatus = modelRF.syncStatusField;
      if (_syncStatus != null) {
        var model = await db.get(modelRF.model, 'select * from ${entry.key}', null, null);
        //await db.get('select * from ${entry.key} where ${_syncStatus} = ?', [0]);
        await _sync(null, model, showLoader: false, config: _configuration);
      }
    }
  }

  onSyncComplete(String serviceName) async {
    Timer.periodic(const Duration(seconds: 3), (timer) async {
      var value = await watchServiceCompleted(serviceName);
      if (value == 1) {
        await pdlg.hide();
        timer.cancel();
      }
    });
  }

  _syncAll() async {
    BuildContext context = Libs.context;
    bool isInternet = await SystemRF.network.checkInternet();
    if (!isInternet) {
      print('Please turn on your internet connection');
      return;
    }
    pdlg = ProgressDialog(
      context,
      type: ProgressDialogType.Download,
      isDismissible: true,
    );
    if (Libs.appModels.get().length > 0) await pdlg.show();

    for (var entry in Libs.appModels.get().entries) {
      var modelRF = entry.value as ModelRF;
      var _syncStatus = modelRF.syncStatusField;
      if (_syncStatus != null) {
        var model = await db.get(modelRF.model, 'select * from ${entry.key} where ${_syncStatus} = ?', [0]);
        print(modelRF.model.runtimeType.toString());
        await _sync(context, model, showLoader: false);
      }
    }
    //Libs.appModels.get().forEach((key, value) {});
    if (Libs.appModels.get().length > 0) await pdlg.hide();
  }

  double percentage = 0;
  _sync(
    BuildContext context,
    Object modelList, {
    Function onSuccess,
    bool showLoader = true,
    String command = 'add',
    Configuration config,
  }) async {
    var itemCount = 0;

    bool isInternet = await SystemRF.network.checkInternet();
    if (!isInternet) {
      print('Please turn on your internet connection');
      return;
    }

    if (showLoader)
      pdlg = ProgressDialog(
        context,
        type: ProgressDialogType.Download,
        isDismissible: true,
      );

    onSyncResponse(Response response, dynamic sid) async {
      if (response.statusCode == 200) {
        var json = jsonDecode(response.body);

        var statusFieldName = (Libs.appModels.get()[json['ModelName']] as ModelRF).syncStatusField ?? 0;
        var uSql =
            'update ${json['ModelName']} set ${statusFieldName} = 1 where ${json['Tag']} = ${json['RecordId']}';
        await db.execute(uSql);

        if (showLoader) await pdlg.hide(); // for add
        // SystemRF.hideKeyboard(context); //??
        if (onSuccess != null) onSuccess(json['Message'] + sid.toString(), 'success', json);
        double per = ((itemCount / (modelList as List).length) * 100).roundToDouble();
        percentage = per == 100.0 ? 0 : per;
      } else {
        if (response.body.isNotEmpty) {
          var json = jsonDecode(response.body);
          onSuccess(json['Message'] + sid.toString(), 'failed');
        } else {
          onSuccess('500: Internal Server Error', 'success');
        }
      }
      if (showLoader) await pdlg.hide();
    }

    for (var item in modelList) {
      itemCount++;
      var tableName = item.runtimeType.toString();
      var packet = Dynamic(item).modelToDataMap();
      var pk = db.getPrimaryKeyName(item);
      var sid = packet[pk];

      packet['UUID'] = sid;
      if (command.contains('add')) packet[pk] = 0;
      syncRequest['TableName'] = tableName;
      syncRequest['Command'] = command;
      syncRequest['ApiPacket'] = {"Packet": packet, "PacketList": null};

      var content = json.encode(syncRequest);

      if (showLoader) await pdlg.show();
      if (showLoader)
        pdlg.update(message: 'Please wait, uploading data... (${tableName})', progress: percentage);

      var apiBaseURL = '${config.http.apiBaseURL}/api/Sync/SendPacket';

      var response = await post(apiBaseURL, body: content, headers: {
        HttpHeaders.contentTypeHeader: "application/json",
      });
      await onSyncResponse(response, sid);
    }
    if (showLoader) await pdlg.hide();
  }

  /*FETCH*/

  var fetchRequest = {
    "UserId": 0,
    "TableName": "Student",
    "Command": "GetData",
    "TotalRecord": 0,
    "PageNo": 1,
    "PageSize": 50,
    "ApiPacket": {},
  };
  var pageSize = 50;
  var currentPage = 1;
  var pages = '';
  var totalRecord = 0;
  List<int> completedPages = List();
  var isOldDelete = '';

  fetchModel(BuildContext context, String serviceName, Function dispatcher,
      [String model, Function onFetchComplete]) async {
    bool isInternet = await SystemRF.network.checkInternet();
    if (!isInternet) {
      rf.msg(context, 'Please turn on your internet connection');
      return;
    }

    currentPage = 1;
    pages = '';
    totalRecord = 0;
    completedPages = List();
    isOldDelete = '';
    pageSize = Libs.config.data.pageSize;
    fetchRequest['PageSize'] = pageSize;

    Map<String, dynamic> inputData = Map();
    inputData['Model'] = model;
    if (dispatcher != null) {
      _fetchWithService(serviceName, context, dispatcher, inputData);
    } else {
      print('Starting Fetch Without Service');
      _fetchWithoutService(context, Models.models[model].model, onFetchComplete); // pass table
    }
  }

  _fetchWithoutService(BuildContext context, dynamic model, Function onFetchComplete) async {
    _fetch(context, model, onFetchComplete: onFetchComplete); // non service
  }

  _fetchWithService(
    String serviceName,
    BuildContext context,
    Function dispatcher,
    Map<String, dynamic> inputData,
  ) async {
    pdlg = ProgressDialog(context, type: ProgressDialogType.Normal, isDismissible: true);

    DialogRF(
      context,
      title: 'Download Data',
      subTitle: 'Download old data from server.',
      avatarIcon: Icons.cloud_download,
      type: DialogType.popup,
      body: Padding(
        padding: EdgeInsets.only(top: 30),
        child: CheckRF(
          data: [
            RadioCheckData(
              label: 'Do you want to delete local data before downloading server data?',
              value: 0,
            )
          ],
          onChange: (val) => isOldDelete = val,
        ),
      ),
      onAccept: () async {
        resetService(serviceName);
        await pdlg.show();
        inputData['DeleteOldData'] = isOldDelete;
        startService(serviceName, dispatcher, inputData);
        onFetchComplete(serviceName);
      },
    );
  }

  runFetchService(Function reflectionAgent, IConfigRF models, Map<String, dynamic> inputData) async {
    reflectionAgent.call();
    Configuration _configuration = new Configuration();
    models.configure(_configuration);
    Models modelList = new Models();
    _configuration.model.appModels.register(modelList);

    if (inputData['DeleteOldData'].isNotEmpty) {
      //same for non service
      print('Deleting old data');
      await db.execute('delete from ${inputData['Model']}');
    }

    for (var entry in modelList.get().entries) {
      var modelRF = entry.value as ModelRF;
      if (inputData['Model'] == modelRF.model.runtimeType.toString()) {
        await _fetch(null, modelRF.model, showLoader: false, config: _configuration, pageNo: 1);
      }
    }
  }

  onFetchComplete(String serviceName) async {
    Timer.periodic(const Duration(seconds: 3), (timer) async {
      var value = await watchServiceCompleted(serviceName);
      if (value == 1) {
        await pdlg.hide();
        timer.cancel();
      }
    });
  }

  //paging + service
  _fetch(
    BuildContext context,
    Object model, {
    int pageNo = 1,
    Function onFetchComplete,
    bool showLoader = true,
    Configuration config,
  }) async {
    percentage = 0; //??

    print('Reading Fetch History');
    await dbHelper.clearFetchHistory(model.runtimeType.toString()); //No Need

    FetchHistory fetchHistory = await dbHelper.readFetchHistory(model.runtimeType.toString());
    if (fetchHistory != null && fetchHistory.Pages != "") {
      var splitted = fetchHistory.Pages.split(',');
      splitted.removeWhere((element) => element == "");
      completedPages = splitted.map((data) => int.parse(data)).toList();
    }
    print('FetchHistory : $fetchHistory');
    await dbHelper.insertFetchHistory(model.runtimeType.toString(), pages, totalRecord, 0); //No Need

    if (showLoader) pdlg = ProgressDialog(context, type: ProgressDialogType.Normal, isDismissible: true);

    var baseUrl = Libs?.config ?? config;
    var apiBaseURL = '${baseUrl.http.apiBaseURL}/api/Fetch/GetPacket';

//    if (fetchHistory != null && fetchHistory.Success == 0) {
//      //if no clear History
//      //else start fetch
//    } //popup for pending fetch
//    else {
//      await dbHelper.clearFetchHistory(model.runtimeType.toString());
//      await dbHelper.insertFetchHistory(model.runtimeType.toString(), pages, totalRecord, 0);
//    }
    //delete old fetch

    if (fetchHistory != null) totalRecord = fetchHistory.TotalRecord;
    print('TotalRecord: $totalRecord');

    if (showLoader) await pdlg.show();

    if (totalRecord == 0) {
      print('Post Request Call: 1');
      if (!completedPages.contains(pageNo)) {
        var response = await postFetchRequest(apiBaseURL, model, 1);
        await processFetchResponse(response, model, 1, onFetchComplete);
      }
    }
    var maxLoop = (totalRecord / pageSize).ceil();
    for (int i = 2; i <= maxLoop; i++) {
      currentPage = i;
      print('Post Request Call: $i');
      if (!completedPages.contains(i)) {
        var response = await postFetchRequest(apiBaseURL, model, i);
        await processFetchResponse(response, model, i, onFetchComplete);
      }
    }
  }

  postFetchRequest(String apiBaseURL, dynamic model, int pageNo) async {
    var modelName = model.runtimeType.toString();
    packet() => {"ModelName": "$modelName", "Select": "", "Where": "", "OrderBy": ""};

    fetchRequest['ApiPacket'] = {"Packet": packet(), "PacketList": null};
    fetchRequest['TableName'] = modelName;
    fetchRequest['PageNo'] = pageNo;

    print('Post Request Call : $pageNo');
    fetchRequest['TotalRecord'] = pageNo == 1 ? 1 : 0;
    print('Request Body $pageNo : $fetchRequest');

    return await post(apiBaseURL, body: json.encode(fetchRequest), headers: {
      HttpHeaders.contentTypeHeader: "application/json",
    });
  }

  processFetchResponse(Response response, dynamic model, int pageNo, Function onFetchComplete) async {
    if (response.statusCode == 200) {
      var json = jsonDecode(response.body);
      print('Api Response : $pageNo');
      print('Api Response $pageNo - JSON : ${json}');

      if (pageNo == 1) totalRecord = json['TotalRecord'];

      var data = json['ApiPacket']['PacketList'] as List;
      print('Total Record : ${data.length}');

      for (var item in data) {
        print('ID : ${item['StudentId']}');
        var newModel = Dynamic(model).newClass();
        item['StudentId'] = null; //??? PK
        Dynamic(newModel).dataMapToModel(item);
        await db.save(newModel);
      }

      completedPages.add(currentPage);
      print('Pages Completed: ${completedPages.toList().toString()}');
      pages += currentPage.toString() + ',';
      currentPage++;
      await dbHelper.updateFetchHistoryPages(
          model.runtimeType.toString(), pages, totalRecord, totalRecord == pageSize ? 1 : 0);
    } else {
      if (response.body.isNotEmpty) {
        var json = jsonDecode(response.body);
        if (onFetchComplete != null) {
          print(json['Message'] + pageNo.toString());
          onFetchComplete(json['Message'] + pageNo.toString(), 'failed', json['ApiPacket']['PacketList']);
        }
      } else {
        print('500: Internal Server Error');
        if (onFetchComplete != null) onFetchComplete('500: Internal Server Error', 'success', null);
      }
      return null;
    }
  }

  remoteGet(Object model, int pageNo, Function onComplete) async {
    //await fetchModel(Libs.context, model, pageNo: pageNo, onComplete: onComplete);
    await fetchModel(Libs.context, null, null, model.runtimeType.toString(), onComplete);
  }

  remoteDelete(Object model, Function onSuccess) async {
    var result;
    var modelList = List();
    modelList.add(model);
    await _sync(Libs.context, modelList, command: 'delete', onSuccess: (m, s, o) {
      result = m;
      if (onSuccess != null) onSuccess(m, s, o);
    });
    return result;
  }

  remoteSave(Object model, String command, Function onSuccess) async {
    var result;
    var modelList = List();
    modelList.add(model);
    await _sync(Libs.context, modelList, command: command, onSuccess: (m, s, o) {
      result = m;
      if (onSuccess != null) onSuccess(m, s, o);
    });
    return result;
  }
}

/*backlog
0. remove g from config
1.      not deliting old DONE (WS Pening)
2.      remove old fetch fn DONE
2.1 non service old delete
3. test bulk sync + fetch (S + WS)
4. remote CRUD
5. if response error ??
6. delete bad sync files & class
*/

//RDB ???????
//BAD CODE
//  onFetchResponse(Response response, dynamic model, bool showLoader, Function onSuccess) async {
//    //responseCount++;
//
//    var fetchItemCount = 0;
//    print('Api Response Status Code : ${response.statusCode}');
//    if (response.statusCode == 200) {
//      var json = jsonDecode(response.body);
//      print('Api Response ${json['PageNo']} - JSON : $json');
//
//      var data = json['ApiPacket']['PacketList'] as List;
//
//      var newModel;
//      print('Total Record : ${data.length}');
//      for (var item in data) {
//        fetchItemCount++; // for upload status percentage
//        print('Record Count : $fetchItemCount');
//        newModel = Dynamic((Libs.appModels.get()[model.runtimeType.toString()] as ModelRF).model).newClass();
//        item['StudentId'] = null; //???
//        Dynamic(newModel).dataMapToModel(item);
//        db.save(newModel);
//
//        double per = ((fetchItemCount / data.length) * 100).roundToDouble();
//        percentage = per == 100.0 ? 0 : per;
//      }
//
//      completedPages.add(json['PageNo']);
//      pages += json['PageNo'].toString() + ',';
//      dbHelper.updateFetchHistoryPages(newModel.runtimeType.toString(), pages, totalRecord, 0);
//
//      if (onSuccess != null)
//        onSuccess(json['Message'] + fetchItemCount.toString(), 'success', json['ApiPacket']['PacketList']);
//
//      print('Pages Completed (Async): ${completedPages.toList().toString()}');
//
//      if ((totalRecord / pageSize).ceil() == completedPages.length) {
//        dbHelper.updateFetchHistorySuccess(newModel.runtimeType.toString(), 1);
//        if (showLoader) await pdlg.hide();
//        print('Fetch Completed');
//      }
//
//      return json['ApiPacket']['PacketList']; // to db
//    } else {
//      if (response.body.isNotEmpty) {
//        var json = jsonDecode(response.body);
//        if (onSuccess != null)
//          onSuccess(json['Message'] + fetchItemCount.toString(), 'failed', json['ApiPacket']['PacketList']);
//      } else {
//        if (onSuccess != null) onSuccess('500: Internal Server Error', 'success', null);
//      }
//      return null;
//    }
//  }

//  dynamic fetch123(
//    BuildContext context,
//    Object model, {
//    bool valid,
//    String loadingMessage,
//    int pageNo = 0,
//    Function onComplete,
//    bool showProgress = true,
//    bool storeWithPaging = false,
//  }) async {
//    if (storeWithPaging) {}
//
//    var params = {
//      "ApiPacket": {
//        "Packet": {
//          "IncludeChild": false,
//          "ModelName": "Student",
//          "OrderBy": "",
//          "Select": "select * from student inner join UserSchool  ON  student.SchoolId=UserSchool.SchoolId",
//          "Where": " UserSchool.UserId = 23 and student.SchoolId=16"
//          // and student.SchoolId=16
//        }
//      },
//      "Command": "GetData",
//      "PageNo": 1,
//      "PageSize": 0,
//      "TableName": "Student",
//      "TotalRecord": 1,
//      "UserId": 23
//    };
//
//    dynamic fetchRND(
//      Object model, {
//      bool valid,
//      String processAlertText,
//      int pageNo = 1,
//      dynamic onComplete,
//    }) async {
//      params['PageNo'] = pageNo;
//
//      var response =
//          await post("http://frmrnd.scibd.info/api/Fetch/GetPacket", body: json.encode(params), headers: {
//        HttpHeaders.contentTypeHeader: "application/json",
//      });
//
//      if (response.statusCode == 200) {
//        var jsonMap = jsonDecode(response.body);
//        //List packetList = jsonMap['ApiPacket']['PacketList'] as List;
////      List<dynamic> records = List();
////      for (int i = 0; i < packetList.length; i++) {
////        var rec =
////            Mapper().jsonToModel(Dynamic(model).newClass(), packetList[i]);
////        records.add(rec);
////      }
//        //onComplete(jsonMap['ApiPacket']['PacketList']);
//        return jsonMap['ApiPacket']['PacketList'];
//      } else {
//        throw Exception('Failed to load album');
//      }
//    }
//  }

///BAD
///

//  ApiResponse abc(dynamic data) {
//    ApiResponse response = ApiResponse();
//
//    response.PageNo = data['PageNo'];
//    var packet = data['ApiPacket']['Packet'];
//    List packetList = data['ApiPacket']['PacketList'] as List;
//
//    response.Success = data['Success']; //??
//    response.Message = data['Message']; //??
//    response.ServerRecordId = data['ServerRecordId']; //
//    response.RecordId = data['RecordId']; //
//    response.TotalRecord = data['TotalRecord']; //
//    response.PageNo = data['PageNo']; //
//    response.PageSize = data['PageSize']; //
//    response.ApiPacket = data['ApiPacket'];
//    response.Status = data['Status']; //
//  }

//class SyncRF {
//  final model;
//  final valid;
//  final handeler;
//  final onComplete;
//  final processAlert;
//
//  SyncRF(this.model, this.handeler, {this.onComplete, this.valid, this.processAlert});
//}
//
//class FetchRF {
//  final model;
//  final valid;
//  final handeler;
//  final onComplete;
//  final processAlert;
//
//  FetchRF(this.model, this.handeler, {this.onComplete, this.valid, this.processAlert});
//}
//
////@Model
//class ApiRequest {
//  int UserId;
//  String TableName;
//  String Command;
//  int TotalRecord;
//  int PageNo;
//  int PageSize;
//  ApiPacket1 ApiPacket = new ApiPacket1();
//}
//
////@Model
//class ApiResponse {
//  bool Success;
//  String Message;
//  int ServerRecordId;
//  Object RecordId;
//  int TotalRecord;
//  int PageNo;
//  int PageSize;
//  ApiPacket1 ApiPacket = new ApiPacket1();
//  int Status;
//}
//
////@Model
//class ApiPacket1<T> {
//  T Packet;
//  List<T> PacketList;
//
//  T getPacket() {
//    return Packet;
//  }
//
//  void setPacket(T packet) {
//    Packet = packet;
//  }
//
//  List<T> getPacketList() {
//    return PacketList;
//  }
//
//  void setPacketList(List<T> packetList) {
//    PacketList = packetList;
//  }
//}
