import 'package:flutter/material.dart';
import 'StoreRF.dart';
import 'uiux/WidgetStylesRF.dart';
import 'uiux/WidgetsRF.dart';
import 'uiux/StyleRF.dart';
import 'RapidFire.dart';

abstract class IConfigRF {
  void configure(Configuration _configuration);
}

abstract class IStoryboardRF {
  StoryboardRF customStoryboard(BuildContext context);
}

abstract class IModelRF {
  void register(Models models);
}

class Configuration {
  AppConfigurationRF app = AppConfigurationRF();
  HttpConfigurationRF http = HttpConfigurationRF();
  ModelsConfigurationRF model = ModelsConfigurationRF();
  PageConfigurationRF page = PageConfigurationRF();
  Data data = Data();
  BackgroundJob service = BackgroundJob();
}

class AppConfigurationRF {
  String title;
  String subTitle;
  TemplateRF template;
  ThemeRF theme;
  IThemeDataRF themeData;

  IPageOutlineRF pageOutline;
  StyleStoreDefaults widgetStyle;
  dynamic onboard; //IAppOnboardRF
}

class HttpConfigurationRF {
  String apiBaseURL = '';
}

class ModelsConfigurationRF {
  IModelRF appModels;
  List<Function> validators;
}

class PageConfigurationRF {
  TransitionTypeRF transition;
  ScrollEffectRF scrollEffect;
}

class Data {
  int pageSize;
  int downloadLimit;
}

class BackgroundJob {
  List<Function> jobs;
  IConfigRF bgConfig;
}

enum ScrollEffectRF { appDefault, elastic }

enum TransitionTypeRF {
  fade,
  rightToLeft,
  leftToRight,
  upToDown,
  downToUp,
  scale,
  rotate,
  size,
  rightToLeftWithFade,
  leftToRightWithFade,
}

class PageImplementationStoreDefaults {
//  final dynamic everything;//
//  final IDrawerScaffoldRF drawer;//
//  final IAppBarScaffoldRF appBar;//
  final IPageOutlineRF outline;

  const PageImplementationStoreDefaults({this.outline} /*this.everything, this.appBar, this.drawer,*/);
}

class Models {
  static Map<String, ModelRF> models = new Map();

  const Models();

  Map<String, Object> get() {
    return models;
  }

  void add(ModelRF model) {
    models[model.model.runtimeType.toString()] = model;
  }
}

class ModelRF {
  Object model;
  String displayName;
  Icon displayIcon;
  bool syncable;
  String syncStatusField;
  List<Object> fetchDependencies;
  bool fetchIncludeChild;
  Function fetchDependencyCallback;

  ModelRF({
    this.model,
    this.displayName,
    this.displayIcon,
    this.syncable = true,
    this.syncStatusField = 'SyncStatus',
    this.fetchDependencies,
    this.fetchIncludeChild = true,
    this.fetchDependencyCallback,
  });
}
