import 'package:flutter/material.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';

import 'Dynamic.dart';

extension HexColor on Color {
  /// String is in the format "aabbcc" or "ffaabbcc" with an optional leading "#".
  Color fromHex(String hexString) {
    final buffer = StringBuffer();
    if (hexString.length == 6 || hexString.length == 7) buffer.write('ff');
    buffer.write(hexString.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }

  /// Prefixes a hash sign if [leadingHashSign] is set to `true` (default is `true`).
  String toHex({bool leadingHashSign = true}) => '${leadingHashSign ? '#' : ''}'
      '${alpha.toRadixString(16).padLeft(2, '0')}'
      '${red.toRadixString(16).padLeft(2, '0')}'
      '${green.toRadixString(16).padLeft(2, '0')}'
      '${blue.toRadixString(16).padLeft(2, '0')}';
}

extension StringColor on String {
  Color get toColor {
    final buffer = StringBuffer();
    final color = this;
    if (color.length == 6 || color.length == 7) buffer.write('ff');
    buffer.write(color.replaceFirst('#', ''));
    return Color(int.parse(buffer.toString(), radix: 16));
  }
}

extension TabBarConverter on TabBarRF {
  TabBar get toTabBar {
    var tab = null;
    if (this is TabBarRF) {
      var _tab = this.tab;
      tab = TabBar(
        controller: _tab.controller,
        tabs: _tab.tabs,
      );
    }
    return tab;
  }
}

extension DynamicExt on Object {
  bool get changed {
    return Dynamic(this).changed1();
  }
}
