library rapidfire;

import 'package:flushbar/flushbar.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_statusbarcolor/flutter_statusbarcolor.dart';
import 'package:page_transition/page_transition.dart';
import 'package:rapidfire/models/AppUser.dart';
import 'package:rapidfire/Api.dart';
import 'package:rapidfire/uiux/StyleRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';
import 'package:shared_preferences/shared_preferences.dart';
//import 'package:rapidfirebgs/rapidfire_bgs.dart';
import 'Config.dart';
import 'Db.dart';
import 'Dynamic.dart';
import 'GlobalRF.dart';
import 'LocalizationRF.dart';
import 'GlobalScopeRF.dart';
import 'GlobalScopeRF.dart' as g;
import 'Rdb.dart';
import 'RestartRF.dart';
import 'package:catcher/catcher.dart';
import 'SystemRFOLD.dart';
import 'package:rapidfire/JobRF.dart';
import 'counter_service.dart';

enum MessageType { info, error, warning }

abstract class IJobWorkerRF {
  execute(String taskName);
}

class RapidFire {
  bool initiated = false;
  IConfigRF _config;
  Configuration _configuration = new Configuration();
  Widget _startPage;
  SharedPreferences pref;
  App app;
  ModelObject model;
  Res lang;
  Db db;
  Rdb rdb;
  Api api;
  Mapper mapper;
  AppStyles style;
  AppOnboardRF onboard;
  PageAction page;
  AppUser appUser;
  BackgroundJobRF job;

  BuildContext context;
  IJobWorkerRF worker;

  // Function jobWorkerFn;

  CatcherOptions debugOptions, releaseOptions;

  RapidFire(IConfigRF config, [Function reflectionAgent, Widget startPage]) {
    if (reflectionAgent != null) reflectionAgent.call();
    WidgetsFlutterBinding.ensureInitialized();

    debugOptions = CatcherOptions(DialogReportMode(), [ConsoleHandler()]);
    releaseOptions = CatcherOptions(DialogReportMode(), [
      EmailManualHandler(["a@b.com"])
    ]);

    _startPage = startPage;
    _config = config;
    //jobWorkerFn = jobWorker;
    run();
  }

  bool started = false;
  Future<dynamic> run([dynamic callback]) async {
    var startPage;
    if (!initiated) {
      _config.configure(_configuration);
      _configuration.model.appModels.register(g.modelList); //==>>>>
      Libs.config = _configuration;

      db = Db(null);
      rdb = Rdb();
      api = Api();
      app = App();
      lang = Res();
      mapper = Mapper();
      style = AppStyles();
      pref = await SharedPreferences.getInstance();

      onboard = AppOnboardRF(
        splash: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.splash),
        language: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.language),
        intro: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.intro),
        login: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.login),
        otp: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.otp),
        setup: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.setup),
        home: AppOnboardRF.clintOnboardWidget(_configuration, OnboardItemsRF.home),
      );

      AppOnboardRF.storeOnboardItems(onboard);

      page = PageAction();
      job = BackgroundJobRF();
      Libs.job = BackgroundJobRF();

      startPage = _startPage ?? onboard.next(); //.splash; //before run

      //db.createTable();
      appUser = AppUser();
      appUser.Name = 'Anawer Hossain';
      appUser.Designation = 'Community Organizer';
      Libs.appUser = appUser;

      rf = this;
      Libs.msg = msg;
    }


    if (callback == null && initiated) {
      initiated = true;
      Catcher(RestartRF(RapidFireAppRF(startPage)), debugConfig: debugOptions, releaseConfig: releaseOptions);
    } else if (callback != null) {
      callback(RestartRF(RapidFireAppRF(startPage)));
    }

    if (!started) {
//      if (Libs.config.service.jobs != null && Libs.config.service.jobs.length > 0){
//        for (var item in Libs.config.service.jobs) {
//          RapidfireBGS.startService(item);
//        }
//        started = true;
//      }
//      RapidfireBGS.startService(Libs.config.service.jobs.first);
//      CounterService().init();
      started = true;
    }

  }

  Flushbar fb;

  msg(BuildContext context, String text,
      {MessageType type = MessageType.info,
      IconData icon,
      Color color = Colors.black87,
      Color backgroundColor = Colors.redAccent,
      String actionText = '',
      Function action}) {
    fb = Flushbar<bool>(
      //title:  null,
      messageText: Padding(
        padding: const EdgeInsets.only(left: 20),
        child: Text(
          text,
          style: TextStyle(
              fontWeight: FontWeight.w500, fontSize: 18.0, color: color, fontFamily: "ShadowsIntoLightTwo"),
        ),
      ),
      duration: Duration(seconds: 5),
      margin: EdgeInsets.all(10),
      padding: EdgeInsets.all(8),
      borderRadius: 50,
      backgroundColor: backgroundColor,
      icon: Icon(
        icon ?? Icons.info,
        color: Colors.white,
        size: 52,
      ),
      mainButton: actionText.isNotEmpty
          ? Padding(
              padding: const EdgeInsets.only(right: 6.0),
              child: FlatButton(
                shape: new RoundedRectangleBorder(borderRadius: new BorderRadius.circular(50.0)),
                color: Colors.white,
                onPressed: () {
                  fb.dismiss(true);
                },
                child: Text(
                  actionText,
                  style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold, fontSize: 18.0),
                ),
              ),
            )
          : null,
    )..show(context).then((value) {
        if (action != null) if (value != null) if (value) action();
      });
  }

// splash logo + app title + app slogan + app color => direct config parm
// intro image, headline + description
// drawer menu items
// switchboard/home => page widget
//var f = AppOnboardRF(home: WidgetsRF.appDrawer());
}

class BackgroundJobRF {
  void run(Function job) async {
   // await RapidfireBGS.startService(job);
  }
}

class PageAction {
  Future call(BuildContext context, dynamic page,
      {bool refresh = false, TransitionTypeRF pageTransition}) async {
//    return Navigator.push(
//      context,
//      new MaterialPageRoute(builder: (context) => page),
//    ).then((value) {
//      if (refresh) PageRF.refresh(context);
//    });

    //    PageTransitionType.downToUp,

    PageTransitionType getPageTransitionType() {
      PageTransitionType type;
      if (pageTransition == null) {
        if (Libs.config.page.transition == null) {
          type = PageTransitionType.downToUp;
        } else {
          type = PageTransitionType.values[Libs.config.page.transition.index];
        }
        ;
      } else {
        type = PageTransitionType.values[pageTransition.index];
      }
      return type;
    }

    var callerStatusBarColor = await FlutterStatusbarcolor.getStatusBarColor();
    var callerNavigationBarColor = await FlutterStatusbarcolor.getNavigationBarColor();

    return Navigator.push(
            context, PageTransition(curve: Curves.linear, type: getPageTransitionType(), child: page))
        .then((value) {
      SystemRFOLD.os.setOsBarColors([callerStatusBarColor, callerNavigationBarColor]);
      if (refresh) {
        PageRF.dataSource?.refresh();
        PageRF.refresh(context);
      }
    });

//    return Navigator.of(context).push(_createRoute(page)).then((value) {
//      if (refresh) PageRF.refresh(context);
//    });
  }

  back(BuildContext context) {
//    Navigator.pushAndRemoveUntil(
//        context,
//        MaterialPageRoute(
//            builder: (context) => page
//        ),
//        ModalRoute.withName("/Home")
//    );

//    return Navigator.pushReplacement(
//      context,
//      new MaterialPageRoute(builder: (context) => page),
//    );

    Navigator.pop(context);
  }

  replace(BuildContext context, dynamic page) {
    return Navigator.pushReplacement(
      context,
      new MaterialPageRoute(builder: (context) => page),
    );
  }

  Route _createRoute(dynamic page) {
    return PageRouteBuilder(
      pageBuilder: (context, animation, secondaryAnimation) => page,
      transitionDuration: Duration(milliseconds: 250),
      maintainState: false,
      transitionsBuilder: (context, animation, secondaryAnimation, child) {
        var begin = Offset(0.0, 1.0);
        var end = Offset.zero;
        var curve = Curves.ease;

        var tween = Tween(begin: begin, end: end).chain(CurveTween(curve: curve));

        return SlideTransition(
          position: animation.drive(tween),
          child: child,
        );
      },
    );
  }
}

class ModelObject {
  dynamic _model;

  ModelObject(dynamic model) {
    _model = model;
  }

  bool changed() {
    return Dynamic(_model).changed1();
  }
}

class Res {
  String text(String name) {
    return LocalizationRF.instance.text(name);
  }
}

class App {
  void restart(BuildContext context) {
    RestartRF.restartApp(context);
  }
}

class Mapper {
  Object jsonToModel(Object obj, Map<String, dynamic> json) {
    Dynamic dynamic = Dynamic(obj);
    dynamic.dataMapToModel(json);
    return obj;
  }
}

class ScreenRF {
  static height(BuildContext context) {
    return MediaQuery.of(context).size.height;
  }

  static width(BuildContext context) {
    return MediaQuery.of(context).size.width;
  }
}

class ClassRF {
  copy() {}

  compare(Object comparableObject) {}
}
