import 'dart:convert';
import 'dart:io';
import 'package:connectivity/connectivity.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:geolocator/geolocator.dart';
import 'package:intl/intl.dart';
import 'package:http/http.dart' as http;
import 'package:rapidfire/Extensions.dart';
import 'package:workmanager/workmanager.dart';

import 'JobRF.dart';

//datetime formater
/*DateTime now = DateTime.now();
String formattedDate = DateFormat('yyyy-MM-dd – kk:mm').format(now);*/

class SystemRF {
  static NetworkRF network = NetworkRF();
  static GPSRF gps = GPSRF();
  static RegisterJobRF job = RegisterJobRF();
  static NotificationRF notification = NotificationRF();
  static hideKeyboard(BuildContext context){
    FocusScope.of(context).requestFocus(FocusNode());
  }
}

class NetworkRF {
  Future<bool> checkWifi() async {
    bool iswifi = false;
    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult == ConnectivityResult.wifi) {
      // I am connected to a wifi network.
      iswifi = true;
    }
    return iswifi;
  }

  Future<bool> checkInternet() async {
    bool isConnected;
    try {
      final result = await InternetAddress.lookup('google.com');
      if (result.isNotEmpty && result[0].rawAddress.isNotEmpty) {
        isConnected = true;
      }
    } on SocketException catch (_) {
      isConnected = false;
    }
    return isConnected;
  }
}

class GPSRF {
  Future<bool> gpsStatus() async {
    bool isLocationEnabled = await Geolocator().isLocationServiceEnabled();
    return isLocationEnabled;
  }
}

class RegisterJobRF {
  killJob(String jobTitle) {}

  register(JobRF1 jobRF) async{
    WidgetsFlutterBinding.ensureInitialized();
    await Workmanager.initialize(jobRF.job, isInDebugMode: false); //to true if still in testing lev turn it to false whenever you are launching the app
    await Workmanager.registerOneOffTask("6", jobRF.name);
  }

//  register(JobRF jobRF) async {
//    //Workmanager.cancelAll();
//   //WidgetsFlutterBinding.ensureInitialized();
////    Workmanager.cancelByUniqueName(jobRF.name);
////    await Workmanager.initialize(jobRF.job, isInDebugMode: false);
////    //to true if still in testing lev turn it to false whenever you are launching the app
////    await Workmanager.registerOneOffTask("6", jobRF.name);
//  }
}

//class JobRF {
//  String name;
//  int period;
//  Function job;
//
//  JobRF({this.name, this.period, this.job});
//}

class NotificationRF {
  notify(NotifyRF notifyRF) async {
    FlutterLocalNotificationsPlugin flp = FlutterLocalNotificationsPlugin();
    var android = AndroidInitializationSettings('@mipmap/ic_launcher');
    var iOS = IOSInitializationSettings();
    var initSetttings = InitializationSettings(android, iOS);
    flp.initialize(initSetttings);
    var androiddetails = AndroidNotificationDetails('channel id', 'channel NAME', 'CHANNEL DESCRIPTION',
        priority: Priority.High, importance: Importance.Max);
    var iOSdetails = IOSNotificationDetails();
    var platform = NotificationDetails(androiddetails, iOSdetails);
    await flp.show(0, notifyRF.title, notifyRF.description, platform, payload: 'VIS \n notifyRF.description');
  }
}

class NotifyRF {
  String title;
  String description;
  String icon;
  dynamic data;
  NotifyRF({this.title, this.description, this.icon, this.data});
}

//region ButtonRF
enum ButtonType { raised, outline, icon, flat, image }
enum ButtonStyleRF { info, danger, warning, submit, cancel, success }

class ButtonRF extends StatefulWidget {
  final ButtonType buttonType;
  final ButtonStyleRF buttonStyle;
  final Text text;
  final Icon icon;
  final Image image;
  final Color color;
  final Color textColor;
  final Color splashColor;
  final double elevation;
  final double width;
  final double height;
  final double borderRadious;
  final Function onTap;
  @override
  _ButtonRFState createState() => _ButtonRFState();
  ButtonRF({
    this.buttonType = ButtonType.raised,
    this.buttonStyle,
    this.text,
    this.icon,
    this.image,
    this.color,
    this.textColor = Colors.black87,
    this.splashColor,
    this.elevation,
    this.width = 100,
    this.height = 45,
    this.borderRadious = 0,
    this.onTap,
  });
}

class _ButtonRFState extends State<ButtonRF> {
  @override
  Widget build(BuildContext context) {
    Color buttonColor = widget.color;
    if (buttonColor == null) {
      switch (widget.buttonStyle) {
        case ButtonStyleRF.info:
          buttonColor = Colors.white;
          break;
        case ButtonStyleRF.danger:
          buttonColor = Colors.red;
          break;
        case ButtonStyleRF.warning:
          buttonColor = Colors.orange;
          break;
        case ButtonStyleRF.submit:
          buttonColor = Colors.blue;
          break;
        case ButtonStyleRF.cancel:
          buttonColor = Colors.red;
          break;
        case ButtonStyleRF.success:
          buttonColor = Colors.green;
          break;
      }
    }

    switch (widget.buttonType) {
      case ButtonType.raised:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: RaisedButton(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[widget.text],
            ),
            onPressed: widget.onTap,
            color: buttonColor,
            elevation: widget.elevation,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;

      case ButtonType.outline:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: OutlineButton(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[widget.text],
            ),
            onPressed: widget.onTap,
            color: widget.color,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            borderSide: BorderSide(color: buttonColor),
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;
      case ButtonType.icon:
        return IconButton(
          icon: new Icon(
            Icons.search,
            color: buttonColor,
          ),
          onPressed: widget.onTap,
          color: buttonColor,
          splashColor: widget.splashColor,
        );
        break;
      case ButtonType.flat:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: FlatButton(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[widget.text],
            ),
            onPressed: widget.onTap,
            color: buttonColor,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;
      case ButtonType.image:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: RaisedButton(
            child: Row(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Expanded(flex: 1, child: widget.image),
                  Expanded(
                      flex: 2, child: Padding(padding: const EdgeInsets.only(left: 10), child: widget.text))
                ]),
            onPressed: widget.onTap,
            color: buttonColor,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;
    }
    return Container();
  }
}
//endregion

//region IncrementRF
enum IncrementType { horizontal, vertical }

class IncrementRF extends StatefulWidget {
  final IncrementType incrementType;

  IncrementRF({this.incrementType = IncrementType.horizontal});

  @override
  _IncrementRFState createState() => _IncrementRFState();
}

class _IncrementRFState extends State<IncrementRF> {
  int counter = 0;
  @override
  Widget build(BuildContext context) {
    switch (widget.incrementType) {
      case IncrementType.vertical:
        return Container(
          child: Row(
            children: <Widget>[
              Container(
                  width: 100,
                  height: 30,
                  margin: EdgeInsets.all(5),
                  decoration: BoxDecoration(
                    color: Colors.white,
                    border: Border.all(width: 1.0, color: Colors.blue),
                    borderRadius:
                        BorderRadius.all(Radius.circular(5.0) //                 <--- border radius here
                            ),
                  ),
                  child: Center(
                      child: Text(
                    counter.toString(),
                    style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold, fontSize: 20),
                  ))),
              Column(
                children: <Widget>[
                  ButtonRF(
                      buttonType: ButtonType.raised,
                      buttonStyle: ButtonStyleRF.info,
                      text: Text("+"),
                      borderRadious: 50,
                      onTap: () {
                        counter++;
                        setState(() {});
                      },
                      width: 40,
                      height: 40),
                  ButtonRF(
                    buttonType: ButtonType.raised,
                    buttonStyle: ButtonStyleRF.info,
                    text: Text(
                      "-",
                      style: TextStyle(fontSize: 20),
                    ),
                    borderRadious: 50,
                    onTap: () {
                      counter--;
                      setState(() {});
                    },
                    width: 40,
                    height: 40,
                  )
                ],
              ),
            ],
          ),
        );
        break;
      case IncrementType.horizontal:
        return Container(
          child: Row(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              ButtonRF(
                  buttonType: ButtonType.raised,
                  buttonStyle: ButtonStyleRF.info,
                  text: Text("+"),
                  borderRadious: 50,
                  onTap: () {
                    counter++;
                    setState(() {});
                  },
                  width: 40,
                  height: 40),
              Container(
                  width: 100,
                  height: 40,
                  margin: EdgeInsets.all(5),
                  decoration: BoxDecoration(
                    color: 'f7f7f7'.toColor, //  border: Border.all(width: 1.0, color: Colors.blue),
                    borderRadius:
                        BorderRadius.all(Radius.circular(5.0) //                 <--- border radius here
                            ),
                  ),
                  child: Center(
                      child: Text(
                    counter.toString(),
                    style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold, fontSize: 20),
                  ))),
              ButtonRF(
                buttonType: ButtonType.raised,
                buttonStyle: ButtonStyleRF.info,
                text: Text(
                  "-",
                  style: TextStyle(fontSize: 20),
                ),
                borderRadious: 50,
                onTap: () {
                  counter--;
                  setState(() {});
                },
                width: 40,
                height: 40,
              )
            ],
          ),
        );
        break;
    }
  }
}
//endregion

// Example
/*class Example {
  test() {
    var internet = SystemRF.network.internetStatus;
    var wifi = SystemRF.network.wifiStatus;
    var gps = SystemRF.gps.gpsStatus;
    SystemRF.job.register(
        JobRF(name: 'myJob', period: null, job: [])
    );
    SystemRF.notification.notify(NotifyRF (title: ""));
  }
}*/
