import 'package:flutter/material.dart';
import 'package:rapidfire/GlobalRF.dart';
import 'package:rapidfire/LocalizationProviderRF.dart';
import 'package:rapidfire/uiux/WidgetsRF.dart';

final List<Language> _languages = <Language>[];

class LanguageSelection extends StatefulWidget {
  @override
  _LanguageState createState() => _LanguageState();
}

class _LanguageState extends State<LanguageSelection> {
  @override
  Widget build(BuildContext context) {
    Language filea = new Language(
      name: "English",
      text: "This is English Language",
      img: 'assets/images/lan-en.png',
    );
    Language fileb = new Language(
      name: "Bangla",
      text: "এটি বাংলা ভাষা",
      img: "assets/images/lan-bg.png",
    );

    setState(() {
      _languages.clear();
      _languages.add(fileb);
      _languages.add(filea);
    });

    return Scaffold(
      backgroundColor: Colors.white,
      body: Container(
        child: Column(
          children: <Widget>[
            SizedBox(
              height: 20,
            ),
            Container(
                margin: EdgeInsets.all(20),
                child: Image(image: AssetImage('assets/images/language.png'))),
            SizedBox(
              height: 10,
            ),
            Text(
              "Select Your Language",
              style: TextStyle(fontSize: 20),
            ),
            SizedBox(
              height: 10,
            ),
            Container(
              //color: Colors.red,
              child: ListView.builder(
                padding: new EdgeInsets.all(8.0),
                shrinkWrap: true,
                reverse: true,
                itemBuilder: (_, int index) => _languages[index],
                itemCount: _languages.length,
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class Language extends StatelessWidget {
  Language({this.text, this.name, this.img});

  final String text;
  final String name;
  final String img;

  List<Widget> languageRow(context) {
    Map<String, String> lag = Map();
    lag['en'] = 'English';
    lag['es'] = 'Bangla';
    return <Widget>[
      Expanded(
        child: InkWell(
          onTap: () {
            switch (name) {
              case 'English':
                LanguageRF.loadLanguage(Locale('en', ''), () {
                  if (AppOnboardRF.currentStep >= 6)
                    Navigator.pop(context);
                  else
                    rf.page.replace(context, rf.onboard.next());
                });
                break;
              case 'Bangla':
                LanguageRF.loadLanguage(Locale('es', ''), () {
                  if (AppOnboardRF.currentStep >= 6)
                    Navigator.pop(context);
                  else
                    rf.page.replace(context, rf.onboard.next());
                });
                break;
            }
          },
          child: Row(
            children: <Widget>[
              CircleAvatar(
                radius: 25 ,
                backgroundImage: AssetImage(img),
                backgroundColor: Colors.transparent,
              ),
              Container(width: 15,),
              Expanded(
                child: new Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    new Text(
                      name,
                      style: TextStyle(
                          color: Colors.black87, fontSize: 16, fontWeight: FontWeight.w800),
                    ),
                    SizedBox(
                      height: 5,
                    ),
                    new Text(
                      text,
                      style: TextStyle(color: Colors.black.withAlpha(400), fontSize: 15),
                    ),
                  ],
                ),
                flex: 70,
              ),
              if (lag[rf.pref.getString("lang")] == name)
                Expanded(
                  child: Padding(
                    padding: const EdgeInsets.only(top:0.0),
                    child: Icon(
                      Icons.check_circle,
                      size: 35,color: Colors.green,
                    ),
                  ),
                  flex: 15,
                )
            ],
          ),
        ),
      ),
    ];
  }

  @override
  Widget build(BuildContext context) {
    return new Container(
      //color: Colors.green,
      margin: const EdgeInsets.symmetric(vertical: 10.0),
      padding: EdgeInsets.all(10),
      child: new Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: languageRow(context),
      ),
    );
  }
}
