import 'dart:async';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import '../Db.dart';

enum DialogType { full, bottom, popup }

class DialogRF {
  final BuildContext context;
  final String title;
  final String subTitle;
  final Widget body;
  final DialogType type;
  final bool hideDefaultControls;
  final Function onAccept;
  final Function onCancel;
  final double height;
  final IconData avatarIcon;
  final double avatarIconSize;
  final bool hideAvatar;

  DialogRF(
    this.context, {
    this.title = "Missing Title",
    this.subTitle = "Missing Sub Title",
    this.body,
    this.type = DialogType.full,
    this.hideDefaultControls = false,
    this.onAccept,
    this.onCancel,
    this.height = 300,
    this.avatarIcon = FontAwesomeIcons.search,
    this.avatarIconSize,
    this.hideAvatar = false,
  }) {
    if (type == DialogType.full)
      Navigator.push(
          context,
          new MaterialPageRoute(
            builder: (BuildContext context) => FullWindowDialog(
              title: title,
              body: body ?? Text('Missing Content'),
              hidePageHeader: hideDefaultControls,
              onAccept: onAccept,
            ),
            fullscreenDialog: false,
          ));
    else if (type == DialogType.popup) {
      showDialog<String>(
        context: context, barrierDismissible: false, // user must tap button!
        builder: (BuildContext context) {
          return PopupWindowDialogRF(
            title: title,
            subTitle: subTitle,
            body: body ?? Text('Missing Content'),
            onAccept: onAccept,
            onCancel: onCancel,
          );
        },
      );
    } else if (type == DialogType.bottom) {
      BottomSheetRF(
        title: title,
        subTitle: subTitle,
        context: context,
        body: body,
        onAccept: onAccept,
        onCancel: onCancel,
        height: height,
        hideDefaultControls: hideDefaultControls,
        avatarIcon: avatarIcon,
        avatarIconSize: avatarIconSize,
        hideAvatar: hideAvatar,
      );
    }
  }
}

//Type 1
class PopupWindowDialogRF extends StatefulWidget {
  final String title;
  final String subTitle;
  final Widget body;
  final Function onAccept;
  final Function onCancel;

  const PopupWindowDialogRF({Key key, this.title, this.subTitle, this.body, this.onCancel, this.onAccept})
      : super(key: key);

  @override
  PopupWindowDialogRFState createState() {
    return new PopupWindowDialogRFState();
  }
}

class PopupWindowDialogRFState extends State<PopupWindowDialogRF> {
  final _resetKey = GlobalKey<FormState>();
  bool _resetValidate = false;
  StreamController<bool> rebuild = StreamController<bool>();

  dialogActions() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      crossAxisAlignment: CrossAxisAlignment.end,
      children: <Widget>[
        FlatButton(
          child: Text(
            'CANCLE',
            style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
          ),
          onPressed: () {
            Navigator.of(context).pop("");
            if (widget.onCancel != null) widget.onCancel.call();
          },
        ),
        FlatButton(
            child: Text(
              'ACCEPT',
              style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
            ),
            onPressed: () {
              Navigator.of(context).pop("");
              if (widget.onAccept != null) widget.onAccept.call();
            })
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Padding(
          padding: EdgeInsets.all(0.0),
          child: Dialog(
            child: SingleChildScrollView(
              child: Form(
                key: _resetKey,
                autovalidate: _resetValidate,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: <Widget>[
                    SizedBox(
                      height: 10,
                    ),
                    Padding(
                      padding: const EdgeInsets.only(left: 12.0),
                      child: Row(
                        children: <Widget>[
                          CircleAvatar(
                            radius: 22,
                            backgroundColor: Colors.blue,
                            child: Icon(
                              Icons.search,
                              size: 34,
                              color: Colors.white,
                            ),
                          ),
                          SizedBox(
                            width: 10,
                          ),
                          Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              Text(widget.title,
                                  style: TextStyle(fontSize: 20.0, fontWeight: FontWeight.bold)),
                              Text(widget.subTitle,
                                  style: TextStyle(
                                      fontSize: 16.0, fontWeight: FontWeight.bold, color: Colors.black54)),
                            ],
                          )
                        ],
                      ),
                    ),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                        padding: EdgeInsets.only(left: 20, top: 10, bottom: 10, right: 20),
                        child: Column(
                          children: <Widget>[
                            widget.body,
                          ],
                        )),
                    SizedBox(
                      height: 10,
                    ),
                    Container(
                      decoration: new BoxDecoration(
                          border: new Border(bottom: new BorderSide(width: 0.6, color: Colors.black))),
                    ),
                    Container(
                      child: dialogActions(),
                      color: Colors.transparent,
                    )
                  ],
                ),
              ),
            ),
          )),
    );
  }
}

//Type 2
class FullWindowDialog extends StatefulWidget {
  final String title;
  final Widget body;
  final bool hidePageHeader;
  final Function onAccept;
  final Function onCancel;

  FullWindowDialog({this.title, this.body, this.hidePageHeader = false, this.onAccept, this.onCancel});

  @override
  FullWindowDialogState createState() => new FullWindowDialogState();
}

class FullWindowDialogState extends State<FullWindowDialog> {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: widget.hidePageHeader
          ? null
          : AppBar(
              leading: GestureDetector(
                onTap: () {
                  //Scaffold.of(context).openDrawer();
                  if (widget.onCancel != null) widget.onCancel.call();
                  Navigator.of(context).pop("");
                },
                child: Icon(Icons.close),
              ),
              title: Text(widget.title),
              actions: [
                new FlatButton(
                  onPressed: () {
                    Navigator.of(context).pop("");
                    if (widget.onAccept != null) widget.onAccept.call();
                  },
                  child: Text(
                    'ACCEPT',
                    style: TextStyle(fontSize: 18.0, fontWeight: FontWeight.bold, color: Colors.white),
                  ),
                ),
              ],
            ),
      body: Container(
        //padding: EdgeInsets.only(left: 20, top: 20, bottom: 20, right: 20),
        alignment: Alignment(0.0, 0.0), child: widget.body, color: Colors.white,
      ),
    );
  }
}

// Type 3
class BottomSheetRF {
  String title;
  String subTitle;
  BuildContext context;
  Widget body;
  Function onAccept;
  Function onCancel;
  double height;
  bool hideDefaultControls;
  final IconData avatarIcon;
  final double avatarIconSize;
  final bool hideAvatar;

  BottomSheetRF({
    this.title,
    this.subTitle,
    this.context,
    this.body,
    this.onAccept,
    this.onCancel,
    this.height = 300,
    this.hideDefaultControls,
    this.avatarIcon = Icons.search,
    this.avatarIconSize = 32,
    this.hideAvatar = false,
  }) {
    showModalBottomSheet<void>(
        isScrollControlled: true,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(0.0),
        ),
        backgroundColor: Colors.transparent,
        context: context,
        builder: (BuildContext context) {
          return Padding(
            padding: EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom),
            child: Container(
              height: height,
              //color: Colors.white,
              child: Padding(
                padding: const EdgeInsets.all(12.0),
                child: Container(
                  color: Colors.transparent,
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Row(
                        children: <Widget>[
                          hideAvatar
                              ? Container()
                              : CircleAvatar(
                                  radius: 24,
                                  backgroundColor: Colors.blue,
                                  child: Icon(
                                    avatarIcon,
                                    size: avatarIconSize,
                                    color: Colors.white,
                                  ),
                                ),
                          SizedBox(
                            width: 10,
                          ),
                          Expanded(
                            child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                Text(
                                  title,
                                  style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
                                ),
                                Text(
                                  subTitle,
                                  style: TextStyle(
                                      fontSize: 16, fontWeight: FontWeight.w600, color: Colors.black54),
                                ),
                              ],
                            ),
                          ),
                          Container(
                            color: Colors.transparent,
                            child: IconButton(
                              padding: EdgeInsets.zero,
                              icon: Icon(
                                Icons.clear,
                                size: 34,
                                color: Colors.black45,
                              ),
                              color: Colors.black,
                              onPressed: () {
                                Navigator.of(context).pop("");
                              },
                            ),
                          )
                        ],
                      ),
                      SizedBox(height: 15),
                      Expanded(child: body),
                      if (!hideDefaultControls) Divider(color: Colors.black54),
                      if (!hideDefaultControls) dialogActions(onAccept, onCancel),
                    ],
                  ),
                ),
              ),
              decoration: BoxDecoration(
                  color: Colors.white,
                  borderRadius:
                      BorderRadius.only(topLeft: Radius.circular(12), topRight: Radius.circular(12))),
            ),
          );
        });
  }

  dialogActions(onAccept, onCancel) {
    return Container(
      height: 40,
      color: Colors.transparent,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: <Widget>[
          FlatButton(
            padding: EdgeInsets.zero,
            child: Text(
              'CANCLE',
              style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
            ),
            onPressed: () {
              Navigator.of(context).pop("");
              if (onCancel != null) onCancel.call();
            },
          ),
          FlatButton(
              padding: EdgeInsets.zero,
              child: Text(
                'ACCEPT',
                style: TextStyle(color: Colors.blue, fontWeight: FontWeight.bold),
              ),
              onPressed: () {
                Navigator.of(context).pop("");
                if (onAccept != null) onAccept.call();
              })
        ],
      ),
    );
  }
}

class DialogItemListTileRF extends StatefulWidget {
  String title;
  String subTitle;
  IconData icon;
  double size;
  Function onTap;

  DialogItemListTileRF({
    this.title,
    this.subTitle,
    this.icon,
    this.size,
    this.onTap,
  });

  @override
  _DialogItemListTileRFState createState() => _DialogItemListTileRFState();
}

class _DialogItemListTileRFState extends State<DialogItemListTileRF> {
  @override
  Widget build(BuildContext context) {
    return FlatButton(
      color: Colors.black12.withOpacity(0.05),
      child: Padding(
        padding: EdgeInsets.only(top: 10, bottom: 10, left: 0, right: 0),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Icon(widget.icon, size: widget.size ?? 50, color: Colors.black45),
            SizedBox(width: 15),
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Text(
                    widget.title,
                    style: TextStyle(fontWeight: FontWeight.bold, fontSize: 20),
                  ),
                  Text(widget.subTitle),
                ],
              ),
            )
          ],
        ),
      ),
      onPressed: () => widget.onTap.call(),
    );
  }
}
