import 'package:flutter/material.dart';
import 'package:searchable_dropdown/searchable_dropdown.dart';

import '../Dynamic.dart';
import '../GlobalScopeRF.dart';
import '../ValidationRF.dart';
import '../Extensions.dart';
import 'WidgetsRF.dart';

class DropdownRF extends StatefulWidget {
  const DropdownRF({
    this.field,
    this.items,
    this.bind,
    this.maxLength,
    this.hintText,
    this.hint,
    this.caption,
    this.onSaved,
    this.validator,
    this.onFieldSubmitted,
  });

  final field;
  final List<DropdownMenuItem> items;
  final dynamic bind;
  final int maxLength;
  final String hintText;
  final String hint;
  final String caption;
  final FormFieldSetter<String> onSaved;
  final FormFieldValidator<String> validator;
  final ValueChanged<String> onFieldSubmitted;

  @override
  _DropdownListRF createState() => _DropdownListRF();
}

class _DropdownListRF extends State<DropdownRF> {
  int selectedValue;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) selectedValue = value;

    var dropdownList = Container(
      color: 'f7f7f7'.toColor,
      child: SearchableDropdown.single(
        items: widget.items,
        value: selectedValue,
        hint: "Select one",
        searchHint: "Select one",
        onChanged: (value) {
          setState(() {
            selectedValue = value;
            Dynamic(model).set(widget.field, selectedValue);
            if (Libs.modelValidationData[widget.field] != null)
              Libs.modelValidationData[widget.field].isValid = true;
          });
        },
        isExpanded: true,
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[dropdownList, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}
