import 'package:catcher/catcher.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:flutter_rating_bar/flutter_rating_bar.dart';
import 'package:flutter_sticky_header/flutter_sticky_header.dart';
import 'package:flutter_xlider/flutter_xlider.dart';
import 'package:intl/intl.dart';
//import 'package:keyboard_visibility/keyboard_visibility.dart';
import 'package:rapidfire/Annotation.dart' as annotation;
import 'package:rapidfire/Config.dart';
import 'package:rapidfire/SystemRFOLD.dart';
import 'package:rapidfire/ValidationRF.dart';
import 'package:rapidfire/LocalizationProviderRF.dart';

import 'package:rapidfire/StoreRF.dart';
import 'package:rapidfire/stock/modern/onboarding/Intro.dart';
import 'package:rapidfire/stock/modern/onboarding/Splash.dart';
import 'package:rapidfire/uiux/intro/Models/page_view_model.dart';
import 'package:rapidfire/uiux/swl/sweetalert.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import '../Db.dart';
import '../Dynamic.dart';
import '../GlobalRF.dart';
import '../LocalizationRF.dart';
import '../RapidFire.dart';
import '../uiux/StyleRF.dart';
import 'AlertRF.dart';
import 'DrawerRF.dart';
import 'WidgetStylesRF.dart';
import '../GlobalScopeRF.dart';
import '../Extensions.dart';

class TabBuilderRF {
  final List<Widget> views;
  final Color selectedLabelColor;
  final Color unselectedLabelColor;
  final TabBarIndicatorSize indicatorSize;
  final Decoration indicatorDecoration;
  final Function onTap;
  final TickerProvider tickerProvider;
  TabController controller;
  List<Widget> tabs;
  Function tabBuilder;
  Function _moveBefore;
  Function _moveAfter;

  moveAfter() {
    _moveAfter();
  }

  moveBefore() {
    _moveBefore();
  }

  TabBuilderRF({
    this.tabs,
    this.views,
    this.controller,
    this.selectedLabelColor,
    this.unselectedLabelColor,
    this.indicatorSize = TabBarIndicatorSize.tab,
    this.indicatorDecoration, // = const BoxDecoration(color: Colors.blue),
    this.onTap,
    this.tickerProvider,
    this.tabBuilder,
  }) {
    _moveBefore = moveBefore;
    _moveAfter = moveAfter;
    //var tabs1 = tabBuilder(0) as List<Widget>;
    if (tabBuilder != null) this.tabs = tabBuilder(0) as List<Widget>;
    this.controller = TabController(length: tabs.length, vsync: tickerProvider);
  }
}

class TabBarRF extends StatefulWidget {
  TabBuilderRF tab;
  TabBuilderRF view;

  // dynamic scaffold;

  TabBarRF({this.tab, this.view});

  @override
  State<StatefulWidget> createState() {
    return _TabBarViewRF();
  }
}

class _TabBarViewRF extends State<TabBarRF> {
  TabController tabController;
  int _selectedIndex = 0;
  int tabLength = 0;

  void initState() {
    if (widget.tab != null) {
      tabController = widget.tab.controller;
      tabLength = widget.tab.tabs.length;
    } else {
      tabController = widget.view.controller;
      tabLength = widget.view.tabs.length;
    }
    tabController.addListener(() {
      setState(() {
        _selectedIndex = tabController.index;
      });
    });
    super.initState();
  }

  __moveBefore() {
    if (_selectedIndex > 0) {
      setState(() => _selectedIndex--);
      tabController.animateTo(_selectedIndex);
    }
  }

  __moveAfter() {
    if (_selectedIndex < (tabLength - 1)) {
      setState(() => _selectedIndex++);
      tabController.animateTo(_selectedIndex);
    }
  }

  @override
  Widget build(BuildContext context) {
    dynamic tabItem;

//    tabController =
//        (tabController == null) ? TabController(length: 1, vsync: widget.tab.tickerProvider) : tabController;

    if (widget.tab != null) {
      widget.tab._moveBefore = __moveBefore;
      widget.tab._moveAfter = __moveAfter;
      if (widget.tab.tabBuilder != null)
        widget.tab.tabs = widget.tab.tabBuilder(_selectedIndex) as List<Widget>;
//      tabController = (tabController == null)
//          ? TabController(length: widget.tab.tabs.length, vsync: widget.tab.tickerProvider)
//          : tabController;
//      widget.tab.controller = tabController;

      tabItem = TabBar(
        labelColor: widget.tab.selectedLabelColor,
        unselectedLabelColor: widget.tab.unselectedLabelColor,
        indicatorSize: widget.tab.indicatorSize,
        isScrollable: true,
        indicator: widget.tab.indicatorDecoration,
        tabs: widget.tab.tabs,
        controller: tabController,
        onTap: (index) {
          setState(() => _selectedIndex = index);
          widget.tab.tabs = widget.tab.tabBuilder(index) as List<Widget>;
          if (widget.tab.onTap != null) widget.tab.onTap(_selectedIndex);
        },
      );
    } else if (widget.view != null) {
      tabItem = Container(
          height: 20,
          child: TabBarView(
            controller: widget.view.controller, //widget.view.controller,
            children: widget.view.views,
          ));
    }
    return tabItem;
  }
}

class TabDesignerRF extends StatefulWidget {
  int index;
  int selected;
  String label;
  Icon icon;
  Decoration decoration;

  TabDesignerRF(
    this.index,
    this.selected, {
    this.label,
    this.icon,
    this.decoration,
  });

  @override
  _TabDesignerRFState createState() => _TabDesignerRFState();
}

class _TabDesignerRFState extends State<TabDesignerRF> {
  @override
  Widget build(BuildContext context) {
    return StoreRF.scaffold().widgets.tabBar.getStyle(widget.index, widget.selected, label: widget.label);
  }
}

//#region RapidFireApp

//#region Onboard
enum OnboardItemsRF { splash, language, intro, login, otp, setup, home }

class AppOnboardRF {
  Widget language;
  dynamic splash;
  dynamic intro;
  Widget login;
  Widget otp;
  Widget setup;
  dynamic home;

  // dynamic drawer;

  AppOnboardRF({
    this.language,
    this.splash,
    @required this.intro,
    this.login,
    this.otp,
    this.setup,
    @required this.home, //   this.drawer,
  });

  static int currentStep = 0;
  static AppOnboardRF onboardItems;
  static List<dynamic> onboardItemList = List();

  static storeOnboardItems(AppOnboardRF items) {
    onboardItems = items;
    onboardItemList.add(onboardItems.splash);
    onboardItemList.add(onboardItems.language);
    onboardItemList.add(onboardItems.intro);
    onboardItemList.add(onboardItems.login);
    onboardItemList.add(onboardItems.otp);
    onboardItemList.add(onboardItems.setup);
    onboardItemList.add(onboardItems.home);
  }

  next() {
    for (int i = currentStep; i < onboardItemList.length; i++) {
      currentStep++;
      if (onboardItemList[i] != null) return onboardItemList[i];
    }
  }

  static onboardWidget(source) {
    if (source is SplashRF)
      return Splash(splash: source);
    else if (source is List<IntroPageViewRF>)
      return Intro(pages: source);
    else
      return source;
  }

  static clintOnboardWidget(
    Configuration _configuration,
    OnboardItemsRF item,
  ) {
    TemplateRF clientTemplate = _configuration.app.template;
    if (_configuration.app.onboard is AppOnboardRF) {
      switch (item) {
        case OnboardItemsRF.splash:
          return onboardWidget(_configuration.app.onboard.splash) ??
              StoreRF.onboard.splash(_configuration, template: clientTemplate);
        case OnboardItemsRF.language:
          return !(_configuration.app.onboard.language is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.language) ??
                  StoreRF.onboard.language(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.intro:
          return !(_configuration.app.onboard.intro is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.intro)
              : null;
        case OnboardItemsRF.login:
          return !(_configuration.app.onboard.login is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.login) ??
                  StoreRF.onboard.login(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.otp:
          return !(_configuration.app.onboard.otp is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.otp) ??
                  StoreRF.onboard.otp(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.setup:
          return !(_configuration.app.onboard.setup is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.setup) ??
                  StoreRF.onboard.setup(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.home:
          assert(_configuration.app.onboard.home != null, 'AppOnboardRF requires Home');
          return _configuration.app.onboard.home;
      }
    } else {
      switch (item) {
        case OnboardItemsRF.splash:
          return onboardWidget(_configuration.app.onboard.customOnboard().splash) ??
              StoreRF.onboard.splash(_configuration, template: clientTemplate);
        case OnboardItemsRF.language:
          return !(_configuration.app.onboard.customOnboard().language is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().language) ??
                  StoreRF.onboard.language(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.intro:
          return !(_configuration.app.onboard.customOnboard().intro is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().intro)
              : null;
        case OnboardItemsRF.login:
          return !(_configuration.app.onboard.customOnboard().login is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().login) ??
                  StoreRF.onboard.login(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.otp:
          return !(_configuration.app.onboard.customOnboard().otp is EmptyRF)
              ? (onboardWidget(_configuration.app.onboard.customOnboard().otp) ??
                  StoreRF.onboard.otp(_configuration, template: clientTemplate))
              : null;
        case OnboardItemsRF.setup:
          return !(_configuration.app.onboard.customOnboard().setup is EmptyRF)
              ? onboardWidget(_configuration.app.onboard.customOnboard().setup) ??
                  StoreRF.onboard.setup(_configuration, template: clientTemplate)
              : null;
        case OnboardItemsRF.home:
          assert(_configuration.app.onboard.customOnboard().home != null, 'AppOnboardRF requires Home');
          return _configuration.app.onboard.customOnboard().home;
      }
    }
  }
}
//#end region OnboardEnd

class StoryboardRF {
  String title; //
  PageOutlineRF outline; //
  FlexBarRF flex; //
  //List<Widget> actions; ////
  Object model;
  List<DataSourceRF> dataSources;
  Widget stickyHeader;
  Widget child;
  final Widget fab; //
  List<DrawerItemRF> drawerItems;
  DrawerItemRF drawerFooter;
  List<Color> osBarColors;

  StoryboardRF(
      {this.title,
      this.outline,
      this.flex,
      this.stickyHeader,
      this.model,
      this.dataSources, //this.actions,
      this.child,
      this.fab,
      this.drawerItems,
      this.drawerFooter,
      this.osBarColors});
}

class EmptyRF extends StatelessWidget {
  static of(Type type, [bool list = false]) {
    if (type == DrawerItemRF) {
      if (list) {
        return List<DrawerItemRF>();
      } else {
        return DrawerItemRF(template: TemplateRF(null, version: 'EmptyRF'));
      }
    } else if (type == List) {
      return List<Widget>();
    }
  }

  @override
  Widget build(BuildContext context) => Container(height: 0, width: 0);
}

//    IPageOutlineRF customAppBar =
//        Libs.config.app.page.everything ?? Libs.config.app.page.outline;

//    if (localOutline?.tabBar?.position == TabPositionRF.top) {
//      tabBar = TabBar(
//        tabs: localOutline.tabBar.tabBar.scaffold.tabs,
//        controller: localOutline.tabBar.tabBar.scaffold.controller,
//      );
//    }
//PageTabBarRF tabBar;

class PageHeaderRF {
  Widget leading;
  List<Widget> actions;
  dynamic drawer;
  double elevation;
  Color backgroundColor;
  Color foregroundColor;

  PageHeaderRF({
    this.leading,
    this.actions,
    this.drawer,
    this.elevation,
    this.backgroundColor,
    this.foregroundColor,
  });
}

enum OutlineDisableItems { header, headerPanel, pageFooter }

class PageOutlineRF {
  PageHeaderRF pageHeader;
  Widget pageHeaderPanel;
  Widget pageFooter;
  EdgeInsets pageHeaderPanelPadding;
  TemplateRF template;

//  style -> bar ->back color, font, elevation
  //pageBar sub class=> leading, actions, drawer, , , ?style?, -disabled, -title?

  PageOutlineRF({
    this.pageHeader,
    this.pageHeaderPanel,
    this.pageFooter,
    this.pageHeaderPanelPadding,
    this.template,
  });

  static PageOutlineRF implemented = PageOutlineRF(); //client, stock,

  static loadPageOutline(template) {
    implemented.pageHeader = Libs.config.app.pageOutline.customOutline().pageHeader ??
        StoreRF.implementation(template).page.outline.customOutline().pageHeader;

    implemented.pageHeader?.drawer = Libs.config.app.pageOutline.customOutline().pageHeader?.drawer ??
        StoreRF.implementation(template).page.outline.customOutline().pageHeader?.drawer;

    implemented.pageHeader?.actions = Libs.config.app.pageOutline.customOutline().pageHeader?.actions ??
        StoreRF.implementation(template).page.outline.customOutline().pageHeader?.actions;

    implemented.pageHeaderPanel = Libs.config.app.pageOutline.customOutline().pageHeaderPanel ??
        StoreRF.implementation(template).page.outline.customOutline().pageHeaderPanel;

    implemented.pageFooter = Libs.config.app.pageOutline.customOutline().pageFooter ??
        StoreRF.implementation(template).page.outline.customOutline().pageFooter;
  }

  static getAppBar(
    BuildContext context,
    PageOutlineRF local,
    String pageTitle,
    FlexBarRF flex,
    EdgeInsets pageHeaderPanelPadding,
    TemplateRF template,
  ) {
    PageOutlineRF _source = PageOutlineRF();
    var appBar;

    // _source.pageHeader = local?.pageHeader ?? implemented.pageHeader;
    _source.pageHeader = PageHeaderRF(
      leading: local?.pageHeader?.leading ?? implemented.pageHeader.leading,
      drawer: local?.pageHeader?.drawer ?? implemented.pageHeader.drawer,
      actions: local?.pageHeader?.actions ?? implemented.pageHeader.actions,
      backgroundColor: local?.pageHeader?.backgroundColor ?? implemented.pageHeader.backgroundColor,
      foregroundColor: local?.pageHeader?.foregroundColor ?? implemented.pageHeader.foregroundColor,
      elevation: local?.pageHeader?.elevation ?? implemented.pageHeader.elevation,
    );
    _source.pageHeaderPanel = local?.pageHeaderPanel ?? implemented.pageHeaderPanel;

    assert(pageTitle != null, "Page title is missing.");

    customAppBar() {
      AppBar pageHeader;
      var _headerPanel = _source?.pageHeaderPanel;
      var pageBottom = PreferredSize(
        child: Container(
          color: Colors.transparent,
          height: 50.0,
          width: ScreenRF.width(context),
          child: Padding(
            padding: EdgeInsets.only(
                top: pageHeaderPanelPadding?.top ?? 5, bottom: pageHeaderPanelPadding?.bottom ?? 5),
            child: _headerPanel is Widget ? _headerPanel : (_headerPanel as TabBarRF).toTabBar,
          ),
        ),
        preferredSize: Size.fromHeight(50.0),
      );

      if (_source?.pageHeader is PageHeaderRF) {
        pageHeader = AppBar(
          elevation: (_source?.pageHeader as PageHeaderRF).elevation,
          backgroundColor: Theme.of(context).primaryColor,
          leading: (_source?.pageHeader as PageHeaderRF).leading is EmptyRF
              ? null
              : (_source?.pageHeader as PageHeaderRF).leading,
          title: Text(pageTitle),
          actions: (_source?.pageHeader as PageHeaderRF).actions,
          bottom: _headerPanel == null ? _headerPanel : pageBottom,
        );
        return pageHeader;
      } else if (_source?.pageHeader is AppBar) {
        pageHeader = _source?.pageHeader as AppBar; //ImroseX
      } else if (_source?.pageHeader is EmptyRF) {
        return null;
      }

      return AppBar(
        leading: (pageHeader)?.leading is EmptyRF ? null : (pageHeader)?.leading,
        automaticallyImplyLeading: (pageHeader)?.automaticallyImplyLeading ?? true,
        title: Text(pageTitle),
        actions: (pageHeader)?.actions,
        flexibleSpace: (pageHeader)?.flexibleSpace,
        bottom: PreferredSize(
            child: Container(
              color: Colors.transparent,
              height: 50.0,
              width: 800,
              child: _headerPanel is Widget ? _headerPanel : (_headerPanel as TabBarRF).toTabBar,
            ),
            preferredSize: Size.fromHeight(50.0)),
        elevation: (pageHeader)?.elevation,
        shape: (pageHeader)?.shape,
        //backgroundColor: (pageHeader)?.backgroundColor,
        brightness: (pageHeader)?.brightness,
        iconTheme: (pageHeader)?.iconTheme,
        actionsIconTheme: (pageHeader)?.actionsIconTheme,
        textTheme: (pageHeader)?.textTheme,
        primary: (pageHeader)?.primary ?? true,
        centerTitle: (pageHeader)?.centerTitle,
        excludeHeaderSemantics: (pageHeader)?.centerTitle ?? false,
        titleSpacing: (pageHeader)?.titleSpacing ?? NavigationToolbar.kMiddleSpacing,
        toolbarOpacity: (pageHeader)?.toolbarOpacity ?? 1.0,
        bottomOpacity: (pageHeader)?.bottomOpacity ?? 1.0,
      );
    }

    // (local?.pageHeader as AppBar).leading = (local?.pageHeader as PageOutlineRF).headerPanel.;

    if (flex == null) {
      appBar = customAppBar();
    }
    return appBar;
  }

  static getAppDrawer(PageOutlineRF local, TemplateRF template) {
    PageOutlineRF _source = PageOutlineRF(
      pageHeader: PageHeaderRF(
        //     leading: (local?.pageHeader as PageHeaderRF)?.leading ??
        //            (implemented.pageHeader as PageHeaderRF).leading,
//        actions: local?.pageHeader is EmptyRF
//            ? null
//            : (local?.pageHeader as PageHeaderRF)?.actions ??
//                (implemented.pageHeader as PageHeaderRF).actions,
        drawer: local?.pageHeader is EmptyRF
            ? null
            : (local?.pageHeader as PageHeaderRF)?.drawer ?? (implemented.pageHeader as PageHeaderRF).drawer,
      ),
    );
    var drawer = (_source.pageHeader as PageHeaderRF).drawer;
    return drawer is EmptyRF ? null : drawer;
  }

//default+local
//static getAppTabBar(){}

  //default check==>>
  static getAppBottomBar(PageOutlineRF local, TemplateRF template) {
    PageOutlineRF _source = PageOutlineRF();
    _source.pageFooter = local?.pageFooter ?? implemented.pageFooter ?? implemented.pageFooter;
    return _source.pageFooter;
  }
}

//class PageTabBarRF {
//  // TabBarRF tabBar;
//  TabBarViewRF items;
//  TabPositionRF position;
//
//  PageTabBarRF({this.items, this.position});
//}

class RapidFireAppRFAncestor extends InheritedWidget {
  Object initialModel;
  DataSourceRF dataSource;
  bool keyboardVisible;

  RapidFireAppRFAncestor({
    Key key,
    this.dataSource,
    this.keyboardVisible = false,
    this.initialModel,
    Widget child,
  }) : super(key: key, child: child);

  @override
  bool updateShouldNotify(RapidFireAppRFAncestor oldWidget) {
    return oldWidget.dataSource != dataSource || oldWidget.keyboardVisible != keyboardVisible;
  }
}

class RapidFireAppRF extends StatefulWidget {
  final dynamic child;

  RapidFireAppRF(this.child);

  static RapidFireAppRFAncestor of(BuildContext context) =>
      context.inheritFromWidgetOfExactType(RapidFireAppRFAncestor) as RapidFireAppRFAncestor;

  @override
  State<StatefulWidget> createState() {
    return _RapidFireAppRF();
  }
}

class _RapidFireAppRF extends State<RapidFireAppRF> {
  LanguageDelegate _localeLanguageOverrideDelegate;

  @override
  void initState() {
    super.initState();
    var lang = rf.pref.getString("lang") ?? 'en';
    _localeLanguageOverrideDelegate = new LanguageDelegate(Locale(lang, ''));
    LanguageRF.loadLanguage = onSetLanguage;
  }

  onSetLanguage(Locale locale, Function onChanged) {
    setState(() {
      _localeLanguageOverrideDelegate = new LanguageDelegate(locale);
      rf.pref.setString('lang', locale.languageCode);
      LanguageRF.onLoadComplete = onChanged;
    });
  }

  @override
  Widget build(BuildContext context) {
    return RapidFireAppRFAncestor(
      child: MaterialApp(
        navigatorKey: Catcher.navigatorKey,
        debugShowCheckedModeBanner: false,
        theme: Libs.config.app.themeData.customThemeData(ThemeDataTypeRF.light),
        darkTheme: Libs.config.app.themeData.customThemeData(ThemeDataTypeRF.dark),
        localizationsDelegates: [
          _localeLanguageOverrideDelegate,
          GlobalMaterialLocalizations.delegate,
          GlobalWidgetsLocalizations.delegate,
        ],
        supportedLocales: LanguageRF.supportedLocales(),
//        localeResolutionCallback:
//            (Locale locale, Iterable<Locale> supportedLocales) {
//          for (Locale supportedLocale in supportedLocales) {
//            if (supportedLocale.languageCode == locale.languageCode ||
//                supportedLocale.countryCode == locale.countryCode) {
//              return supportedLocale;
//            }
//          }
//          return supportedLocales.elementAt(0);
//        },
        home: widget.child,
      ),
    );
  }
}

//  final dynamic appBar;
//  final dynamic drawer;
//  final TabBarRF tabBar;
//  final TabPositionRF tabBarPosition;
//  final dynamic bottomBar;
//      this.appBar,
//      this.drawer,
//      this.tabBar,
//      this.tabBarPosition,
//      this.bottomBar,
// final Widget body;
//final Widget leading;
// this.body,

class PageBuilderRF extends StatefulWidget {
  final Function(BuildContext) builder;

  const PageBuilderRF({Key key, this.builder}) : super(key: key);

  @override
  PageBuilderRFState createState() => new PageBuilderRFState();

  static PageBuilderRFState of(BuildContext context) {
    return context.ancestorStateOfType(const TypeMatcher<PageBuilderRFState>());
  }
}

class PageBuilderRFState extends State<PageBuilderRF> {
  @override
  Widget build(BuildContext context) {
    return widget.builder(context);
  }

  void rebuild() {
    setState(() {});
  }
}

class PageAncestorRF extends InheritedWidget {
  final Object model;
  FlexBarRF flex;

  PageAncestorRF({
    Key key,
    this.model,
    this.flex,
    Widget child,
  }) : super(key: key, child: child);

  @override
  bool updateShouldNotify(PageAncestorRF oldWidget) {
    return oldWidget.model != model || oldWidget.flex != flex;
  }
}

class PageRF extends StatefulWidget {
  final String title;
  final PageOutlineRF outline;
  final Object model;
  final Function loadModel;
  final FlexBarRF flex;
  final stickyHeader;
  final Widget body;
  final Widget fab; //fabEx + on click
  final FloatingActionButtonAnimator fabEx; //fabEx + on click
  final Widget filter;
  final Function onBack;
  final Color backgroundColor;
  final bool disabledPageScroll;
  final List<Color> osBarColors;
  TemplateRF template;

  PageRF({
    this.title,
    this.outline,
    this.model,
    this.loadModel,
    this.flex,
    this.stickyHeader,
    this.body,
    this.fab,
    this.fabEx,
    this.filter,
    this.onBack,
    this.backgroundColor,
    this.disabledPageScroll = false,
    this.osBarColors,
    this.template,
  });

  //static int currentPage = 1;

  static DataSourceRF dataSource;

  static PageAncestorRF of(BuildContext context) =>
      context.inheritFromWidgetOfExactType(PageAncestorRF) as PageAncestorRF;

  static refresh(BuildContext context) {
    (context as Element).reassemble();
  }

  static List<Function> _rebuild = List();
  static rebuild() {
    _rebuild[0].call();
  }

  static bool isModelValid(BuildContext context) {
    bool valid = true;

    Libs.modelValidationFunctions.forEach((key, value) {
      var ret = value.call();
      if (ret != null) {
        if (!ret) valid = false;
      }
    });
    if (!valid) rf.msg(context, 'Data save was unsuccessful due to missing mandatory fields!');
    return valid;
  }

  @override
  State<StatefulWidget> createState() => _PageRF(osBarColors);
}

class _PageRF extends State<PageRF> {
  List<Color> _colors;

  dynamic modelPKValue, initialModelPKValue;

  _PageRF(List<Color> colors) {
    _colors = colors;
  }

  @override
  void initState() {
    //currentPage = 1;
    widget.template = widget.template ?? TemplateRF(TemplateNameRF.modern);

    PageRF.dataSource = null;
    Libs.modelValidationData.clear();
    Libs.modelValidationFunctions.clear();

    SystemRFOLD.os.setOsBarColors(_colors);

//    KeyboardVisibilityNotification().addNewListener(
//      onChange: (bool visible) {
//        RapidFireAppRF.of(context).keyboardVisible = visible;
//        print(visible);
//      },
//    );
    super.initState();
  }

  @override
  void didChangeDependencies() {
    if (widget.model != null) {
      ValidationRF.validationAnnotations = Dynamic(widget.model).annotationData(annotation.Check);

      var _initialModel = RapidFireAppRF.of(context).initialModel = Dynamic(widget.model).copy();

      initialModelPKValue = Db(null).getPrimaryKeyValue(_initialModel);
    }
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    Libs.context = context;

    if (widget.model != null) {
      if (initialModelPKValue == null) {
        modelPKValue = Db(null).getPrimaryKeyValue(widget.model);
        if (modelPKValue != null && modelPKValue > 0) {
          RapidFireAppRF.of(context).initialModel = Dynamic(widget.model).copy();
        }
      }
    }

    PageOutlineRF.loadPageOutline(widget.template);

    var page = PageAncestorRF(
      model: widget.model,
      flex: widget.flex,
      child: PageBuilderRF(builder: (context) {
        return Scaffold(
          appBar: PageOutlineRF.getAppBar(
            context,
            widget.outline,
            widget.title,
            widget.flex,
            widget.outline?.pageHeaderPanelPadding,
            widget.template,
          ),
          drawer: PageOutlineRF.getAppDrawer(
            widget.outline,
            widget.template,
          ),
          body: WillPopScope(
            onWillPop: widget.onBack ??
                () {
                  if (widget.model != null) {
                    if (Dynamic(widget.model).changed(RapidFireAppRF.of(context).initialModel)) {
                      rf.msg(
                        context,
                        'Save change pending, do you want to quite anyway?',
                        icon: Icons.error,
                        actionText: 'Quit',
                        action: () => Navigator.pop(context),
                      );
                    }
                  } else {
                    Navigator.pop(context);
                  }
                  return Future.value(true);
                },
            child: widget.flex == null
                ? flatPage(widget.title, widget?.stickyHeader, widget.disabledPageScroll)
                : flexPage(widget.flex, widget.title, widget.stickyHeader, widget.disabledPageScroll),
          ),
          floatingActionButton: widget.fab,
          floatingActionButtonAnimator: widget.fabEx,
          bottomNavigationBar: widget.outline?.pageFooter,
          //PageOutlineRF.getAppBottomBar(widget.outline, widget.theme),
        );
      }),
    );

    return page;
  }

  SliverAppBar sliverAppBar(BuildContext context, Widget leading, FlexBarRF flex, String title) {
    var titleCenter = true;
    var top = 0.0;
    return SliverAppBar(
      iconTheme: IconThemeData(
        color: Colors.black,
      ),
      leading: leading is EmptyRF ? null : leading,
      expandedHeight: 230.0,
      floating: true,
      backgroundColor: Colors.white,
      actions: widget?.outline?.pageHeader?.actions,
      pinned: true,
      elevation: 0,
      flexibleSpace: LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
        // print('constraints=' + constraints.toString());
        top = constraints.biggest.height;
        if (top <= 80) {
          titleCenter = false;
        } else {
          titleCenter = true;
        }
        return FlexibleSpaceBar(
            centerTitle: titleCenter,
            title: AnimatedOpacity(
              duration: Duration(milliseconds: 100), opacity: top == 80.0 ? 1.0 : 0.0, //opacity: 1.0,
              child: Text(
                title, //widget.flex.title,
                style: TextStyle(fontSize: 20.0, color: Colors.black),
              ),
            ),
            background: Column(
              children: <Widget>[
                Container(height: 220, padding: EdgeInsets.only(top: 20), child: flex.background),
                Padding(
                  padding: const EdgeInsets.only(bottom: 8.0),
                  child: Text(flex.label,
                      style: TextStyle(
                        fontSize: 20.0,
                        color: Colors.black,
                        fontWeight: FontWeight.bold,
                      )),
                ),
              ],
            ));
        //Image.network(widget.flex.background, fit: BoxFit.cover));
      }),
    );
  }

  pageContent(FlexBarRF flex, String title, Widget stickyHeader, bool disablePageScroll) {
    var content = CustomScrollView(
      physics: disablePageScroll ? NeverScrollableScrollPhysics() : null,
      slivers: <Widget>[
        if (flex != null)
          sliverAppBar(
            context, widget?.outline?.pageHeader?.leading, // ?? PageOutlineRF.implemented.pageHeader.leading,
            flex, title,
          ),
        SliverStickyHeader(
          header: Container(
            height: stickyHeader == null ? 0.0 : (flex == null ? 64 : 56),
            color: Colors.white,
            padding: EdgeInsets.only(top: 0.0),
            alignment: Alignment.center,
            child: Padding(
              padding: EdgeInsets.only(top: (flex == null ? 10 : 6)),
              child: stickyHeader,
            ),
          ),
          sliver: SliverList(delegate: SliverChildListDelegate([widget.body])),
        ),
        //SliverList(delegate: SliverChildListDelegate([widget.body]))
      ],
    );

    if (Libs.config.page.scrollEffect == null) {
      return ScrollConfiguration(
        behavior: BounceScrollBehavior(),
        child: content,
      );
    } else {
      if (Libs.config.page.scrollEffect == ScrollEffectRF.elastic) {
        return ScrollConfiguration(
          behavior: BounceScrollBehavior(),
          child: content,
        );
      } else {
        return content;
      }
    }
  }

  Scaffold flexPage(FlexBarRF flex, String title, Widget stickyHeader, bool disablePageScroll) {
    return Scaffold(
      backgroundColor: widget.backgroundColor ?? Colors.white,
      body: GestureDetector(
        onTap: () {
          var f = 0;
        },
        child: NotificationListener<ScrollNotification>(
          onNotification: (scrollNotification) {
//          if (scrollNotification.metrics.pixels >= scrollNotification.metrics.maxScrollExtent && !scrollNotification.metrics.outOfRange) {
            if (scrollNotification.metrics.maxScrollExtent == scrollNotification.metrics.pixels) {
              // if scrollNotification.metrics.pixels < 0
              if (!RapidFireAppRF.of(context).keyboardVisible) {
                var ds = PageRF.dataSource;
                if (ds != null) if (!ds.isLoading)
                  ds.read(
                    ds.sql,
                    ds.currentPage++,
                    DataSourceReadType.auto,
                  );
              }
            }
            return;
          },
          child: pageContent(flex, title, stickyHeader, disablePageScroll),
        ),
      ),
    );
  }

  Widget flatPage(String title, Widget stickyHeader, bool disablePageScroll) {
    return NotificationListener<ScrollNotification>(
        onNotification: (scrollNotification) {
          if (scrollNotification.metrics.maxScrollExtent == scrollNotification.metrics.pixels) {
            if (!RapidFireAppRF.of(context).keyboardVisible) {
              var ds = PageRF.dataSource;
              if (ds != null) if (!ds.isLoading)
                ds.read(
                  ds.sql,
                  ds.currentPage++,
                  DataSourceReadType.auto,
                );
            }
          }
          return;
        },
        child: pageContent(null, title, stickyHeader, disablePageScroll));
  }
}
//#endregion

//#region Page

class FlexBarRF {
  var label;
  var background;
  var color;

  // var stickyHeader;

  FlexBarRF({this.label, this.background, this.color});
}

//#endregion Page End

//#region ContainerRF
class ContainerRF extends StatefulWidget {
  final AlignmentGeometry alignment;

  const ContainerRF({
    this.alignment,
    this.padding,
    this.margin,
    this.height,
    this.decoration,
    this.child,
    this.template,
    this.style,
  });

  final EdgeInsetsGeometry padding;
  final EdgeInsetsGeometry margin;
  final double height;
  final Decoration decoration;
  final Widget child;
  final dynamic template;
  final StyleRF style;

  @override
  _ContainerRF createState() => _ContainerRF();
}

//IContainerStyleRF clint =
//    g.clientWidgetStyle?.everything ?? g.clientWidgetStyle.containerStyleRF;
class _ContainerRF extends State<ContainerRF> {
  @override
  Widget build(BuildContext context) {
    var currentTheme = widget.template ?? Libs.config.app.template ?? TemplateRF.get();
    IContainerRFStyle clint = Libs.config.app.widgetStyle.everything ?? Libs.config.app.widgetStyle.container;

    StyleContainerRF containerStyle =
        clint.customStyleContainerRF() ?? StoreRF.style(currentTheme).container.customStyleContainerRF();

    var alignment = widget.alignment ?? containerStyle.alignment;
    var padding = widget.padding ?? containerStyle.padding;
    var margin = widget.margin ?? containerStyle.margin;
    var decoration = widget.decoration ?? containerStyle.decoration;
    decoration = BoxDecoration(
      color: (decoration as BoxDecoration).color,
      image: (decoration as BoxDecoration).image,
      border: (decoration as BoxDecoration).border,
      borderRadius: (decoration as BoxDecoration).borderRadius,
      boxShadow: widget.style?.shadow ?? (decoration as BoxDecoration).boxShadow,
      gradient: (decoration as BoxDecoration).gradient,
      backgroundBlendMode: (decoration as BoxDecoration).backgroundBlendMode,
      shape: BoxShape.rectangle,
    );
    return Container(
      alignment: alignment,
      padding: padding,
      margin: margin,
      child: widget.child,
      decoration: decoration, //==>> remove StyleContainerRF to StyleRF
    );
  }
}
//#endregion ContainerRF

//#region TextRF
class TextRF extends StatefulWidget {
  final String text;
  final double size;
  final FontWeight weight;
  final Color color;
  final TextStyle style;
  final bool visible;
  final dynamic template;

  const TextRF(
    this.text, {
    this.size,
    this.weight,
    this.color,
    this.visible = true,
    this.style,
    this.template,
  });

  @override
  _TextRF createState() => _TextRF();
}

//ITextStyle clintTextRF =
//    g.clientWidgetStyle?.everything ?? g.clientWidgetStyle.textStyleRF;
class _TextRF extends State<TextRF> {
  @override
  Widget build(BuildContext context) {
    ITextRFStyle clintTextRF = Libs.config.app.widgetStyle.everything ?? Libs.config.app.widgetStyle.text;

    StyleTextRF textStyle =
        clintTextRF.customStyleTextRF() ?? StoreRF.style(widget.template).text.customStyleTextRF();

    var fontSize = widget.size ?? textStyle.fontSize;
    var fontWeight = widget.weight ?? textStyle.fontWeight;
    var color = widget.color ?? textStyle.color;
    return Visibility(
        visible: widget.visible,
        child: Text(
          rf.lang.text(widget.text).replaceAll('not found', ''),
          style: widget.style ??
              TextStyle(
                fontSize: fontSize,
                fontWeight: fontWeight,
                color: color,
              ),
        ));
  }
}
//#endregion TextRF

//#region RowRF
class RowRF extends StatefulWidget {
  const RowRF(this.controls, [this.flex, this.x = 25, this.y = 10]);

  final dynamic controls;
  final double x;
  final double y;
  final List<int> flex;

  @override
  _RowRF createState() => _RowRF();
}

class _RowRF extends State<RowRF> {
  @override
  Widget build(BuildContext context) {
    int count = 1;
    var widgets = List<Widget>();

    for (var item in widget.controls) {
      var _flex;
      if (widget.flex != null) {
        _flex = widget?.flex[count - 1];
      }
      var ctl = Expanded(child: item, flex: _flex ?? 100);
      //print(_flex.toString());
      widgets.add(ctl);
      if (widget.controls.length != count) widgets.add(SizedBox(width: widget.y));
      count++;
    }
    if (widget.x != 0) {
      widgets.add(Container(
        height: widget.x,
      ));
    }

    return Row(
      mainAxisAlignment: MainAxisAlignment.start,
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: widgets,
    );
  }
}
//#endregion RowRF

//region TextFieldRF
enum ValidationTypeRF { required, range, max, min }

class TextFieldRF extends StatefulWidget {
  TextFieldRF({
    this.model,
    this.field,
    this.inputType,
    this.maxLength,
    this.hintText,
    this.hint,
    this.caption,
    this.onSaved,
    this.validator,
    this.onFieldSubmitted,
    this.validator1,
    this.onChange,
  });

  final Object model;
  dynamic field;
  final TextInputType inputType;
  final int maxLength;
  final String hintText;
  final String hint;
  final String caption;

  final FormFieldSetter<String> onSaved;
  final FormFieldValidator<String> validator;
  final ValueChanged<String> onFieldSubmitted;

  final Function validator1;
  final Function onChange;

  @override
  _TextFormFieldRF createState() => _TextFormFieldRF();
}

class _TextFormFieldRF extends State<TextFieldRF> {
  TextEditingController controller = TextEditingController();
  FocusNode _focus = new FocusNode();

  @override
  void initState() {
    _focus.addListener(_onFocusChange);
    super.initState();
  }

  void _onFocusChange() {
    if (!_focus.hasFocus) {
      setState(() {
        if (Libs.modelValidationData[widget.field] != null)
          Libs.modelValidationData[widget.field].isValid = true;
        controller.selection = TextSelection.fromPosition(TextPosition(offset: controller.text.length));
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    model = model ?? widget.model;

    if (widget.field != null) {
      Libs.modelValidationFunctions[widget.field] = () {
        if (widget.field != null) {
          var value = Dynamic(model).get(widget.field);
          ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
          setState(() {
            Libs.modelValidationData[widget.field] = data;
          });
          return data?.isValid;
        }
        return false;
      };
    }

    var fieldValue = model == null ? null : Dynamic(model).get(widget.field);
    if (fieldValue != null) {
      controller..text = fieldValue.toString();
    }

    var textField = Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        TextField(
          controller: controller,
          focusNode: _focus,
          keyboardType: widget.inputType,
          onChanged: (input) {
            if (model != null) Dynamic(model).set(widget.field, input.trim());
            if (widget.onChange != null) widget.onChange(input);
          },
          decoration: new InputDecoration(
            fillColor: 'f7f7f7'.toColor,
            border: InputBorder.none,
            focusColor: Colors.transparent,
            filled: true,
            hintText: widget.hint,
            //hintStyle: new TextStyle(color: Colors.black54),
            //prefix: Text('   '), //suffix: Text('   '),
            //contentPadding: EdgeInsets.only(left: 0, right: 0, top: 0),
          ),
        ),
        ValidationRF.showErrorMessage(widget.field)
      ],
    );

    return textField;
  }
}
//endregion TextFieldRF

//region DatePickerRF
class DatePickerRF extends StatefulWidget {
  String field;
  DateTime maxDate;
  DateTime minDate;

  @override
  _DatePickerRFState createState() => _DatePickerRFState();

  DatePickerRF({this.field, this.maxDate, this.minDate}) {
    if (maxDate == null) maxDate = DateTime(2025);
    if (minDate == null) minDate = DateTime(2000);
  }
}

class _DatePickerRFState extends State<DatePickerRF> {
  String dateValue = 'Select Date';

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field) as String;
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) dateValue = value.toString();

    var datePicker = Container(
      height: 45,
      color: 'f7f7f7'.toColor,
      child: InkWell(
        child: Center(
          child: Text(
            dateValue,
            style: TextStyle(color: Colors.grey, fontWeight: FontWeight.bold),
          ),
        ),
        onTap: () async {
          var dateFormat = DateFormat('yyyy-MM-dd');
          DateTime initalDate = DateTime.now();
          initalDate = DateTime.parse(dateFormat.format(initalDate));
          final DateTime picked = await showDatePicker(
              context: context, initialDate: initalDate, firstDate: widget.minDate, lastDate: widget.maxDate);
          setState(() {
            dateValue = dateFormat.format(picked);
            Dynamic(model).set(widget.field, dateValue);
            if (Libs.modelValidationData[widget.field] != null)
              Libs.modelValidationData[widget.field].isValid = true;
          });
        },
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(constraints: BoxConstraints.expand(height: 45), child: datePicker),
        ValidationRF.showErrorMessage(widget.field)
      ],
    );
  }
}
//endregion DatePickerRF

//region TimePickerRF
class TimePickerRF extends StatefulWidget {
  String field;

  @override
  _TimePickerRFState createState() => _TimePickerRFState();

  TimePickerRF({this.field});
}

class _TimePickerRFState extends State<TimePickerRF> {
  String timeValue = 'Select Time';
  DateTime maxDate;
  DateTime minDate;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field) as String;
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) timeValue = value.toString();

    var timePicker = Container(
      height: 45,
      color: 'f7f7f7'.toColor,
      child: InkWell(
        child: Center(
            child: Text(
          timeValue,
          style: TextStyle(color: Colors.grey, fontWeight: FontWeight.bold),
        )),
        onTap: () async {
          var myFormat = DateFormat('yyyy-MM-dd');
          DateTime initalDate = DateTime.now();
          initalDate = DateTime.parse(myFormat.format(initalDate));
          final selectedTime = await showTimePicker(
            initialTime: TimeOfDay.now(),
            context: context,
          );
          setState(() {
            timeValue = selectedTime.toString();
            Dynamic(model).set(widget.field, timeValue);
            if (Libs.modelValidationData[widget.field] != null)
              Libs.modelValidationData[widget.field].isValid = true;
          });
        },
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(constraints: BoxConstraints.expand(height: 45), child: timePicker),
        ValidationRF.showErrorMessage(widget.field)
      ],
    );
  }
}

//region TimePickerRF

//region SliderRF
class SliderRF extends StatefulWidget {
  final String field;

  @override
  _SliderRFState createState() => _SliderRFState();

  SliderRF({this.field});
}

class _SliderRFState extends State<SliderRF> {
  var currentValue = [0.0];
  var _lowerValue, _upperValue;

  @override
  Widget build(BuildContext context) {
    double width = 18;

    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) currentValue = [value];

    var slider = Container(
      margin: const EdgeInsets.only(right: 15),
      //color: Colors.grey,
      constraints: BoxConstraints.expand(height: 40),
      child: Stack(
        children: <Widget>[
          Positioned.fill(
            top: width,
            bottom: width,
            child: Container(
              //color: Colors.yellow,
              height: 5, //width: MediaQuery.of(context).size.width,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: <Widget>[
                  Container(
                    width: 20,
                    height: 3.5,
                    color: Colors.blue,
                    child: Text(' '),
                  ),
                  Container(
                    width: 20,
                    height: 3.5,
                    color: Color(0xfeeeeff),
                    child: Text(' '),
                  )
                ],
              ),
            ),
          ),
          Positioned(
            child: FlutterSlider(
              trackBar: FlutterSliderTrackBar(),
              values: currentValue,
              max: 500,
              min: 0,
              onDragging: (handlerIndex, lowerValue, upperValue) {
                _lowerValue = lowerValue;
                _upperValue = upperValue;
                setState(() {
                  currentValue[0] = lowerValue;
                  print(currentValue[0]);
                  Dynamic(model).set(widget.field, currentValue[0]);
                  if (Libs.modelValidationData[widget.field] != null)
                    Libs.modelValidationData[widget.field].isValid = true;
                });
              },
            ),
          ),
        ],
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[slider, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}

//endregion SliderRF

//region RatingRF
class RatingRF extends StatefulWidget {
  final String field;

  @override
  _RatingRFState createState() => _RatingRFState();

  RatingRF({this.field});
}

class _RatingRFState extends State<RatingRF> {
  double initialRating = 0;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) initialRating = value;

    var rating = Container(
      height: 40,
      child: Stack(
        overflow: Overflow.visible,
        children: <Widget>[
          Positioned.fill(
            top: 2,
            left: -10,
            child: RatingBar(
              initialRating: initialRating,
              minRating: 1,
              direction: Axis.horizontal,
              allowHalfRating: true,
              itemCount: 5,
              itemSize: 30,
              itemPadding: EdgeInsets.symmetric(horizontal: 4.0),
              itemBuilder: (context, _) => Icon(
                Icons.star,
                color: Colors.amber,
              ),
              onRatingUpdate: (rating) {
                print(rating);
                Dynamic(model).set(widget.field, rating);
                setState(() {
                  if (Libs.modelValidationData[widget.field] != null)
                    Libs.modelValidationData[widget.field].isValid = true;
                });
              },
            ),
          ),
        ],
      ),
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[rating, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}

//endregion RatingRF

//region SwitchRF
class SwitchRF extends StatefulWidget {
  final String field;

  @override
  _SwitchRFState createState() => _SwitchRFState();

  SwitchRF({this.field});
}

class _SwitchRFState extends State<SwitchRF> {
  int isSwitched = 0;

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    var value = Dynamic(model).get(widget.field);
    if (value != null) isSwitched = value;

    var toggleSwitch = Row(
      children: <Widget>[
        Text('Active'),
        SizedBox(
          width: 35,
        ),
        Switch(
          value: isSwitched == 0 ? false : true,
          onChanged: (value) {
            setState(() {
              isSwitched = value ? 1 : 0;
              Dynamic(model).set(widget.field, isSwitched);
              if (Libs.modelValidationData[widget.field] != null)
                Libs.modelValidationData[widget.field].isValid = true;
            });
          },
//          activeTrackColor: Colors.lightGreenAccent,
//          activeColor: Colors.green,
        ),
      ],
    );

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[toggleSwitch, ValidationRF.showErrorMessage(widget.field)],
    );
  }
}

//endregion SwitchRF

class RadioCheckData {
  final String label;
  final dynamic value;

  RadioCheckData({this.label, this.value});
}

class RadioRF extends StatefulWidget {
  final String field;
  final int value;
  final List<RadioCheckData> data;
  final dynamic dimension;
  final bool scrollable;
  final double height;
  final Function onChange;
  final bool separator;
  final double paddingTop;

  @override
  _RadioRFState createState() => _RadioRFState();

  RadioRF({
    this.data,
    this.dimension,
    this.field,
    this.scrollable,
    this.height,
    this.value,
    this.onChange,
    this.separator = false,
    this.paddingTop = 0,
  });
}

class _RadioRFState extends State<RadioRF> {
  int selectedValue = -1; // value could be String or anything ?
  double baseHeight = 40;

  @override
  void didChangeDependencies() {
    var model = PageRF.of(context)?.model;

    var value = (widget.field == null)
        ? widget.value ?? null
        : (model == null) ? null : Dynamic(model).get(widget.field);

    //var value = Dynamic(model).get(widget.field);
    if (value != null) selectedValue = value.runtimeType == int ? value : int.parse(value);
    if (widget.onChange != null) widget.onChange(value);
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    setSelectedRadio(int val) {
      setState(() {
        selectedValue = val;
        if (model != null) Dynamic(model).set(widget.field, selectedValue);
        if (widget.onChange != null) widget.onChange(selectedValue.toString());
        if (Libs.modelValidationData[widget.field] != null)
          Libs.modelValidationData[widget.field].isValid = true;
      });
    }

    var stack = Stack(
      overflow: Overflow.visible,
      children: <Widget>[],
    );
    var _baseHeight = generateRadioList(widget.data, selectedValue, setSelectedRadio, stack);

    setState(() {
      baseHeight = _baseHeight;
    });

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(constraints: BoxConstraints.expand(height: baseHeight), child: stack),
        ValidationRF.showErrorMessage(widget.field)
        //Text('test')
      ],
    );
  }

  double generateRadioList(dynamic _data, dynamic _selectedValue, _fn, dynamic _stack) {
    double _height = 0;
    double offsetTop;
    double offsetTopHeight = 40;

    _stack.children.clear();
    for (RadioCheckData item in _data) {
      if (offsetTop == null)
        offsetTop = -10;
      else
        offsetTop =
            offsetTop + offsetTopHeight + (widget.separator ? (item == _data[_data.length - 1] ? 8 : 16) : 2);

      var pos = Positioned(
          top: offsetTop,
          left: -11,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              SizedBox(height: widget.paddingTop),
              Row(
                children: <Widget>[
                  Radio(
                      materialTapTargetSize: MaterialTapTargetSize.shrinkWrap,
                      value: item.value,
                      groupValue: _selectedValue,
                      onChanged: (value) {
                        _fn(value);
                        print(value);
                      }),
                  //Text(item.label, style: TextStyle(fontWeight: FontWeight.bold))
                  CheckRadioLabelRF(
                    index: item.value,
                    label: item.label,
                    onChange: (value) {
                      _fn(int.parse(value));
                      print(value.toString());
                    },
                  )
                ],
              ),
              Container(height: 5),
              widget.separator
                  ? item == _data[_data.length - 1]
                      ? Container()
                      : Container(
                          height: 1,
                          color: Colors.black12,
                          width: MediaQuery.of(context).size.width - 100,
                        )
                  : Container(),
            ],
          ));

      _height = _height + offsetTopHeight;
      _stack.children.add(pos);
    }
    return _height + 10; // 10 for click last item
  }
}
//endregion RadioGroup

//region CheckboxGroup
class CheckRF extends StatefulWidget {
  final List<RadioCheckData> data;
  final dynamic dimension;
  final String field;
  final String value;
  final bool scrollable;
  final double height;
  final Function onChange;
  final bool separator;
  final double paddingTop;

  @override
  _CheckRFState createState() => _CheckRFState();

  CheckRF({
    this.data,
    this.dimension,
    this.field,
    this.value,
    this.scrollable,
    this.height,
    this.onChange,
    this.separator = false,
    this.paddingTop = 0,
  });
}

class _CheckRFState extends State<CheckRF> {
  Map<int, bool> currentValueMap = Map();
  double baseHeight = 40;

  @override
  void didChangeDependencies() {
    var model = PageRF.of(context)?.model;

    // set value
    var value = (widget.field == null)
        ? widget.value ?? null
        : (model == null) ? null : Dynamic(model).get(widget.field);

    if (value != null) {
      Map<int, bool> currentValueMapFromString = Map();
      for (var item in value.toString().split(RegExp(','))) {
        if (!item.isEmpty) currentValueMapFromString[int.parse(item)] = true;
      }
      currentValueMap = currentValueMapFromString;
      if (widget.onChange != null) widget.onChange(value);
    }
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context)?.model;

    Libs.modelValidationFunctions[widget.field] = () {
      if (widget.field != null) {
        var value = Dynamic(model).get(widget.field);
        ValidationDataRF data = ValidationRF.validate(widget.field, model, value);
        setState(() {
          Libs.modelValidationData[widget.field] = data;
        });
        return data?.isValid;
      }
      return false;
    };

    // on Select
    setSelectedRadio(int value, bool val) {
      var string = '';
      setState(() {
        currentValueMap[value] = val;
        currentValueMap.forEach((k, v) {
          if (v) string += '${k.toString()},';
        });
        print(string);
        if (model != null) Dynamic(model).set(widget.field, string);
        if (widget.onChange != null) widget.onChange(string);
        if (Libs.modelValidationData[widget.field] != null)
          Libs.modelValidationData[widget.field].isValid = true;
      });
    }

    var stack = Stack(
      overflow: Overflow.visible,
      children: <Widget>[],
    );
    var _baseHeight = generateCheckList(widget.data, currentValueMap, setSelectedRadio, stack);

    setState(() {
      baseHeight = _baseHeight;
    });

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Container(constraints: BoxConstraints.expand(height: baseHeight), child: stack),
        ValidationRF.showErrorMessage(widget.field)
      ],
    );
  }

  double generateCheckList(List<RadioCheckData> _data, Map<int, bool> _currentValueMap, _fn, dynamic _stack) {
    double _height = 0;
    double offsetTop;
    double offsetTopHeight = 40;

    _stack.children.clear();
    for (RadioCheckData item in _data) {
      if (offsetTop == null)
        offsetTop = -10;
      else
        offsetTop = offsetTop + offsetTopHeight + (widget.separator ? 10 : 2);

      var pos = Positioned(
          top: offsetTop,
          left: -14,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              SizedBox(height: widget.paddingTop),
              Row(
                children: <Widget>[
                  Checkbox(
                      value: _currentValueMap[item.value] ?? false,
                      onChanged: (newValue) => _fn(item.value, newValue)),
                  //Text(item.label, style: TextStyle(fontWeight: FontWeight.bold))
                  CheckRadioLabelRF(
                    index: item.value,
                    label: item.label,
                    onChange: (value) {
                      _fn(value, _currentValueMap[value] == null ? false : !_currentValueMap[value]);
                    },
                  )
                ],
              ),
              widget.separator
                  ? item == _data[_data.length - 1]
                      ? Container()
                      : Container(
                          height: 1,
                          color: Colors.black12,
                          width: MediaQuery.of(context).size.width - 100,
                        )
                  : Container(),
            ],
          ));

      _height = _height + offsetTopHeight;
      _stack.children.add(pos);
    }
    return _height + 10; // 10 added for last item clickable
  }
}

class CheckRadioLabelRF extends StatefulWidget {
  int index;
  String label;
  TextStyle style;
  Function onChange;

  CheckRadioLabelRF({this.index, this.label, this.style, this.onChange});

  @override
  _CheckRadioLabelRFState createState() => _CheckRadioLabelRFState();
}

class _CheckRadioLabelRFState extends State<CheckRadioLabelRF> {
  @override
  Widget build(BuildContext context) {
    return Material(
      child: InkWell(
        onTap: () {
          widget.onChange(widget.index.toString());
        },
        child: Container(
          color: Colors.white,
          width: 280,
          child: Padding(
            padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
            child: TextRF(widget.label, style: widget.style ?? TextStyle(fontWeight: FontWeight.bold)),
          ),
        ),
      ),
    );
  }
}

//CheckboxGroup(
//orientation: GroupedButtonsOrientation.VERTICAL,
//padding: EdgeInsets.only(left: 0),
//margin: EdgeInsets.all(0),
//labels: <String>[
//"Sports",
//"Reading",
//],
//onSelected: (List<String> checked) => print(checked.toString()),
//
//)
//endregion CheckboxGroup

//region ListViewRF

dialofg(BuildContext context, Function fn) {
  SweetAlertOptions options = new SweetAlertOptions(
    showCancelButton: true,
    title: 'title',
    subtitle: 'subtitle',
    style: SweetAlertStyle.confirm,
    onPress: fn,
  );

  return new Container(
    color: Colors.transparent,
    child: new Padding(
      padding: new EdgeInsets.all(40.0),
      child: new Scaffold(
        backgroundColor: Colors.transparent,
        body: new SweetAlertDialog(curve: null, options: options),
      ),
    ),
  );
}

class ListViewRF extends StatefulWidget {
  DataSourceRF dataSource;
  final dynamic model;
  final dynamic builder;
  final dynamic decoration;
  final Function onTap;
  final Function listBuilder;

  ListViewRF({
    this.dataSource,
    this.model,
    this.builder,
    this.decoration,
    this.onTap,
    this.listBuilder,
  });

  @override
  _ListViewRF createState() => _ListViewRF(dataSource);
}

class _ListViewRF extends State<ListViewRF> {
  ScrollController scrollController = ScrollController();
  DataSourceRF __dataSource;
  bool isLoading = false;
  var dataList = List();
  int currentPage = 1;

  _ListViewRF(DataSourceRF _dataSource) {
    __dataSource = _dataSource;
    _dataSource?.stateUpdateFn = updateState;
    _dataSource?.readListItemFn = getDataList;
    _dataSource?.deleteListItemFn = deleteItem;
    _dataSource?.updateLoadingStateFn = updateLoadingState;
  }

  @override
  void initState() {
    super.initState();
    __dataSource.read(__dataSource.sql, 0, DataSourceReadType.none); // this._fetchPage(1, 20);
// List Scroll
//    scrollController.addListener(() {
//      if (scrollController.position.maxScrollExtent == scrollController.position.pixels) {
//        //_fetchPage(currentPage++, 20);
//        isLoading = true;
//        __dataSource.readData(__dataSource.sql, currentPage++);
//      }
//    });
  }

  updateLoadingState(bool value) {
    setState(() => isLoading = value);
  }

  deleteItem(int index) {
    setState(() => dataList.removeAt(index));
  }

  getDataList() {
    return dataList;
  }

  updateState(dynamic data, int pageNo) {
    setState(() {
      if (pageNo == 0) dataList.clear();
      if (data != null) dataList.addAll(data as List);
      print('updateState: Page No: ${pageNo}');
    });
  }

  delete1Item(int index) {
    setState(() => dataList.removeAt(index));
  }

//  _fetchPage(int pageNumber, int pageSize) async {
  //  if (!isLoading) {
//      setState(() {
//        isLoading = true;
//      });
  //var data = widget.dataSource;
  //var data = await api.fetch(widget.dataSource, pageNo: pageNumber);
//      var data = await rf.db.get(
//          widget.model, 'SELECT * FROM Student ORDER BY StudentId DESC');
//      setState(() {
//        dataList.addAll(data as List);
//        isLoading = false;
//      });
  //   }
  // }

  Widget progressIndicator() {
    return Container(
      color: Colors.white,
      height: 60,
      width: 100,
      child: new Padding(
        padding: const EdgeInsets.all(8.0),
        child: new Center(
          child: new Opacity(
            opacity: isLoading ? 1.0 : 0.0,
            child: //LoadingIndicator(indicatorType: Indicator.ballPulse, color: Colors.pink,)
                CircularProgressIndicator(),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
//  RapidFireAppRF.of(context).dataSource = __dataSource;
    PageRF.dataSource = __dataSource;
    return MediaQuery.removePadding(
      context: context,
      removeTop: true,
      child: ListView.builder(
        shrinkWrap: PageRF.of(context).flex == null ? false : true,
        controller: scrollController,
        itemCount: dataList.length,
        itemBuilder: (BuildContext context, int index) {
          var model = dataList[index];
          var pkValue = 0;

          if (dataList.length > 0) {
            if (model is Map) {
              var newModel;
              Map<dynamic, dynamic> newModelMap = model;
              ModelRF mrf = Libs.appModels.get()[__dataSource.model.toString()];
              newModel = Dynamic(mrf.model).newClass();
              newModelMap.forEach((key, value) {
                Dynamic(newModel).set(key, value);
              });
              model = newModel;
            }

            var pk = Dynamic(model).annotationData(annotation.Key).first['field'];
            pkValue = Dynamic(model).get(pk);
          }

          if (index == dataList.length) {
            return progressIndicator();
          } else {
            return Dismissible(
              key: Key(pkValue.toString()),
              confirmDismiss: (direction) async {
                return await AlertRF(
                    context: context,
                    title: 'Confirm',
                    icon: Icons.error,
                    msg: 'Do you want to delete this record?',
                    confirmText: 'DELETE',
                    onConfirm: (value) {
                      if (value) {
                        if (__dataSource.type == DataSourceType.local) {
                          rf.db.delete(model, pkValue);
                        } else {
                          rf.rdb.delete(model, pkValue);
                        }
                        print('deleting...${pkValue}');
                      }
                    }).show();
              },
              background: Container(color: Color(0xFff1616)),
              child: InkWell(
                onTap: () => widget.onTap(model),
                child: widget.builder(model, index),
              ),
            );
          }
        },
      ),
    );
  }
}

class KeepAliveFutureBuilder extends StatefulWidget {
  final Future future;
  final AsyncWidgetBuilder builder;

  KeepAliveFutureBuilder({this.future, this.builder});

  @override
  _KeepAliveFutureBuilderState createState() => _KeepAliveFutureBuilderState();
}

class _KeepAliveFutureBuilderState extends State<KeepAliveFutureBuilder> with AutomaticKeepAliveClientMixin {
  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: widget.future,
      builder: widget.builder,
    );
  }

  @override
  bool get wantKeepAlive => true;
}
//https://medium.com/saugo360/flutter-creating-a-listview-that-loads-one-page-at-a-time-c5c91b6fabd3
//endregion ListViewRF

const gridPadding = EdgeInsets.all(10);

class GridViewRF extends StatefulWidget {
  final dynamic itemList;
  final dynamic child;
  final EdgeInsetsGeometry padding;
  final dynamic decoration;
  final int column;
  final double spaceX;
  final double spaceY;
  final bool isScroll;
  final GridBuild build;

  const GridViewRF(
      {this.itemList,
      this.child,
      this.padding = gridPadding,
      this.decoration,
      this.column = 2,
      this.spaceX = 2.0,
      this.spaceY = 2.0,
      this.isScroll = true,
      this.build});

  @override
  _GridViewRF createState() => _GridViewRF();
}

typedef GridBuild = Widget Function(BuildContext context, int index);

class _GridViewRF extends State<GridViewRF> {
  @override
  Widget build(BuildContext context) {
    dynamic modelList = widget.itemList;

    return Container(
        padding: widget.padding,
        child: GridView.builder(
          physics: !widget.isScroll ? const NeverScrollableScrollPhysics() : null,
          itemCount: modelList.length,
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: widget.column, crossAxisSpacing: widget.spaceY, mainAxisSpacing: widget.spaceX),
          itemBuilder: (BuildContext context, int index) {
            return widget.build(context, index);
          },
        ));
  }
}
//endregion

class FilterRF extends StatefulWidget {
  Function onAdvanceFilter;
  DataSourceRF dataSource;
  Color color;
  String quickFilterFields;
  String defaultSorting;

  @override
  _FilterRFState createState() => _FilterRFState();

  FilterRF(
      {this.dataSource,
      this.onAdvanceFilter,
      this.quickFilterFields,
      this.defaultSorting = 'DESC',
      this.color});
}

class _FilterRFState extends State<FilterRF> {
  var quickFilterValue;
  final textController = TextEditingController();
  @override
  Widget build(BuildContext context) {
    return Container(
      width: MediaQuery.of(context).size.width - 25,
      height: 55,
      child: Row(
        children: <Widget>[
          Expanded(
            // Quick Search Text Box
            child: Padding(
              padding: const EdgeInsets.only(left: 12, top: 0),
              child: TextField(
                controller: textController,
                decoration: InputDecoration(border: InputBorder.none, hintText: 'Enter a search term'),
                onChanged: (input) {
                  setState(() => quickFilterValue = input);
                },
              ),
            ),
            flex: 70,
          ),
          Expanded(
            // Quick Filter
            child: IconButton(
              icon: FaIcon(FontAwesomeIcons.search, size: 20, color: Colors.blue),
              onPressed: () {
                widget.dataSource.currentPage = 1;
                FocusScope.of(context).requestFocus(FocusNode());
                widget.dataSource.read(
                  '${widget.quickFilterFields}#${quickFilterValue}#${widget.defaultSorting}',
                  0,
                  DataSourceReadType.quick,
                );
              },
            ),
            flex: 10,
          ),
          Expanded(
            child: IconButton(
              //Advance Filter
              icon: FaIcon(FontAwesomeIcons.slidersH, size: 20, color: Colors.blue),
              onPressed: () => {
                widget.dataSource.currentPage = 1,
                widget.onAdvanceFilter(),
              },
            ),
            flex: 10,
            //FaIcon(FontAwesomeIcons.slidersH, size: 20, color: Colors.blue,), flex: 10,
          ),
          Expanded(
            //Reset list
            child: IconButton(
              icon: FaIcon(FontAwesomeIcons.redo, size: 20, color: Colors.blue),
              onPressed: () {
                widget.dataSource.currentPage = 1;
                widget.dataSource.whereFields = '';
                widget.dataSource.read(widget.dataSource.initSql, 0, DataSourceReadType.none);
                textController.text = '';
              },
            ),
            flex: 10,
          ),
          SizedBox(width: 5)
        ],
      ),
      decoration: BoxDecoration(
          color: widget.color ?? Colors.white, //shape: BoxShape.circle,
          boxShadow: [
            BoxShadow(
              color: Colors.grey.withOpacity(0.4),
              blurRadius: 5.0,
              spreadRadius: 2.0,
            ),
          ],
          borderRadius: BorderRadius.all(Radius.circular(8))),
    );
  }
}

class CardActionRF extends StatefulWidget {
  final icon;
  final deleteId;
  final index;
  final size;
  final color;
  final onAction;
  DataSourceRF dataSource;
  List<Function> modelValidators;

  @override
  _CardActionRFState createState() => _CardActionRFState();

  CardActionRF(this.icon, this.deleteId, this.dataSource, this.index,
      {this.color, this.size, this.onAction, this.modelValidators});
}

class _CardActionRFState extends State<CardActionRF> {
  @override
  Widget build(BuildContext context) {
    var model = PageRF.of(context).model;
    return IconButton(
        icon: Icon(widget.icon),
        iconSize: 40,
        color: Color.fromRGBO(140, 140, 140, 1),
        onPressed: () {
          AlertRF(
              context: context,
              title: 'Confirm',
              icon: Icons.error,
              msg: 'Do you want to delete this record?',
              confirmText: 'DELETE',
              onConfirm: (value) async {
                if (value) {
                  var ret = await rf.db.delete(model, widget.deleteId, widget.modelValidators);
                  if (ret == 0) {
                    rf.msg(context, 'Synced data can not be deleted');
                  } else {
                    widget.dataSource.delete(widget.index);
                    PageRF.refresh(context);
                  }
                }
              }).show();
        }
        //widget.onAction,
        );
  }
}

class CardAvatarRF extends StatefulWidget {
  @required
  final image;
  final borderColor;
  final borderWidth;
  final statusColor;

  @override
  _CardAvatarRFState createState() => _CardAvatarRFState();

  CardAvatarRF({
    this.image,
    this.borderColor,
    this.borderWidth,
    this.statusColor = Colors.green,
  });
}

class _CardAvatarRFState extends State<CardAvatarRF> {
  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        Container(
          width: 50.0,
          height: 50.0,
          decoration: new BoxDecoration(
            image: new DecorationImage(
              image: AssetImage(widget.image),
              fit: BoxFit.cover,
            ),
            borderRadius: new BorderRadius.all(new Radius.circular(50.0)),
            border: new Border.all(
              color: Colors.red,
              width: 2.0,
            ),
          ),
        ),
        Positioned(
          bottom: 0,
          right: 0,
          child: CircleAvatar(
            backgroundColor: Colors.white,
            radius: 9,
            child: CircleAvatar(
              backgroundColor: widget.statusColor,
              radius: 7,
            ),
          ),
        )
      ],
    );
  }
}

class ListCardRF extends StatefulWidget {
  final leading;
  final body;
  final action;

  @override
  _ListCardRFState createState() => _ListCardRFState();

  ListCardRF({this.leading, this.body, this.action});
}

class _ListCardRFState extends State<ListCardRF> {
  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(left: 10.0, right: 8.0, top: 8, bottom: 8),
          child: Row(
            children: <Widget>[
              widget.leading,
              Expanded(
                child: Padding(
                  padding: EdgeInsets.only(left: 18.0, top: 5),
                  child: widget.body,
                ),
                flex: 70,
              ),
              Expanded(
                child: widget.action,
                flex: 15,
              ),
            ],
          ),
        ),
        //        ListTile(
//          leading: AvatarRF(),
//          title: Text('Name : ${studentName}, ID : ${studentId}',
//            style: TextStyle(fontWeight: FontWeight.bold),
//          ),
//          subtitle: Text('Enroll Date: ${enroleDate}'),
//          trailing: IconButton(
//            icon: Icon(Icons.delete_sweep),
//            iconSize: 40,
//            onPressed: () {
//              rf.db.delete(Student(), data.StudentId);
//              PageRF.refresh(context);
//            },
//          ),
//        ),
        Container(height: 1, color: Colors.black12)
      ],
    );
  }
}

class GridCardRF extends StatefulWidget {
  int index;
  List<String> cardTitleList;
  List<IconData> cardImageList;
  Function onTap;

  GridCardRF({
    this.index,
    this.cardTitleList,
    this.cardImageList,
    this.onTap,
  });

  @override
  _GridCardRFState createState() => _GridCardRFState();
}

class _GridCardRFState extends State<GridCardRF> {
  @override
  Widget build(BuildContext context) {
    return FlatButton(
      onPressed: () => widget.onTap(widget.index),
      color: Colors.white,
      child: Container(
        //color: Colors.red,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            FaIcon(widget.cardImageList[widget.index], size: 40, color: Colors.pink),
            SizedBox(height: 5),
            Center(
              child: Text(
                widget.cardTitleList[widget.index].toUpperCase(),
                textAlign: TextAlign.center,
                style: TextStyle(fontWeight: FontWeight.bold, fontSize: 15),
              ),
            )
          ],
        ),
      ),
    );
  }
}

class LabelRF extends StatefulWidget {
  final text;

  @override
  _LabelRFState createState() => _LabelRFState();

  LabelRF(this.text);
}

class _LabelRFState extends State<LabelRF> {
  @override
  Widget build(BuildContext context) {
    return TextRF(
      widget.text,
      style: TextStyle(fontWeight: FontWeight.bold, fontSize: 14),
    );
  }
}

class PageTitleRF extends StatefulWidget {
  final text;
  @override
  _PageTitleRFState createState() => _PageTitleRFState();
  PageTitleRF(this.text);
}

class _PageTitleRFState extends State<PageTitleRF> {
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 40,
      child: Stack(
        overflow: Overflow.visible,
        children: <Widget>[
          Positioned(
            left: -10,
            child: Container(
                child: Padding(
                  padding: const EdgeInsets.only(left: 5, top: 5, bottom: 5),
                  child: Text(
                    widget.text,
                    style: TextStyle(
                      color: Colors.red,
                      fontSize: 20,
                      fontWeight: FontWeight.bold,
                      fontFamily: 'Roboto',
                      letterSpacing: 1,
                      decoration: TextDecoration.none,
                    ),
                  ),
                ),
                decoration: BoxDecoration(
                    border: Border(
                        left: BorderSide(
                  color: Colors.deepOrange,
                  width: 8,
                )))),
          )
        ],
      ),
    );
  }
}

class PageSubTitleRF extends StatefulWidget {
  final text;
  @override
  _PageSubTitleRFState createState() => _PageSubTitleRFState();
  PageSubTitleRF(this.text);
}

class _PageSubTitleRFState extends State<PageSubTitleRF> {
  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.only(bottom: 15.0),
      child: Text(
        widget.text,
        style: TextStyle(
          color: Colors.black54,
          fontSize: 18,
          fontWeight: FontWeight.bold,
          fontFamily: 'Roboto',
          letterSpacing: 1,
          decoration: TextDecoration.none,
        ),
      ),
    );
  }
}

enum GapTypeRF { Gap5, Gap10 }

class GapRF extends StatefulWidget {
  int gap;

  GapRF([this.gap = 5]);

  @override
  _GapRFState createState() => _GapRFState();
}

class _GapRFState extends State<GapRF> {
  @override
  Widget build(BuildContext context) {
    return Container(
      height: 5,
    );
  }
}

Widget fab(Function onPressed) {
  return FloatingActionButton(
    backgroundColor: Colors.red,
    elevation: 10,
    onPressed: () {
      onPressed();
      //navigateToDetail(Profile(), 'Add Profile');
    },
    tooltip: 'Add',
    child: Icon(
      Icons.add,
      color: Colors.white,
    ),
  );
}

class BounceScrollBehavior extends ScrollBehavior {
  @override
  getScrollPhysics(_) => const BouncingScrollPhysics();
}

class WidgetsRF {
  static selectWidget(local, clint, store) {
    var widget = Object();
    if (local is EmptyRF) {
      widget = null;
    } else if (local.runtimeType.toString().lastIndexOf('RF') > 1) {
      if (local is TextRF || local is ContainerRF || local is DrawerRF) {
        if (local?.template?.version == 'EmptyRF') {
          widget = null;
        }
      }
    }

    if (clint is EmptyRF) {
      widget = null;
    } else if (clint.runtimeType.toString().lastIndexOf('RF') > 1) {
      if (clint is TextRF || clint is ContainerRF || clint is DrawerRF) {
        if (clint?.template?.version == 'EmptyRF') {
          widget = null;
        }
      }
    }

    if (widget == null) {
      return null;
    } else {
      return local ?? clint ?? store;
    }
  }
}

class SplashRF {
  String title;
  String subTitle;
  Color color;
  String font;
  Color titleColor;
  Color subTitleColor;
  Color rippleColor;
  String rippleImagePath;
  String orgLogoPath;
  int duration;

  SplashRF({
    this.title,
    this.subTitle,
    this.color,
    this.font,
    this.titleColor,
    this.subTitleColor,
    this.rippleColor,
    this.rippleImagePath,
    this.orgLogoPath,
    this.duration = 5,
  });
}

enum RippleEffectRF { style1 }

class IntroRF {
  Widget title;
  Widget image;
  Widget description;
  Widget pointerImage;
  Color backgroundColor;
  RippleEffectRF rippleEffect;
  Color rippleColor;

  IntroRF({
    this.title,
    this.image,
    this.description,
    this.pointerImage,
    this.backgroundColor,
    this.rippleEffect,
    this.rippleColor,
  });
}
